@@ -19,7 +19,7 @@ namespace aqua
             int lines;
             std::vector<double> timestamps;
             void work(ccsds::CCSDSPacket &packet);
-            image::Image<uint16_t> getChannel(int channel);
+            image::Image getChannel(int channel);
         };
     } // namespace amsu
 } // namespace aqua
\ No newline at end of file
