@@ -11,6 +11,14 @@
 #include "products/dataset.h"
 #include "resources.h"
 #include "instruments/modis/modis_histmatch.h"
+#include "nlohmann/json_utils.h"
+
+#include "common/calibration.h"
+#include "instruments/modis/calibrator/modis_calibrator.h"
+#include "core/exception.h"
+
+#include "common/image/io.h"
+#include "common/image/image_utils.h"
 
 namespace eos
 {
@@ -27,7 +35,7 @@ namespace eos
             else if (parameters["satellite"] == "aura")
                 d_satellite = AURA;
             else
-                throw std::runtime_error("EOS Instruments Decoder : EOS satellite \"" + parameters["satellite"].get<std::string>() + "\" is not valid!");
+                throw satdump_exception("EOS Instruments Decoder : EOS satellite \"" + parameters["satellite"].get<std::string>() + "\" is not valid!");
         }
 
         void EOSInstrumentsDecoderModule::process()
@@ -143,7 +151,7 @@ namespace eos
                 if (time(NULL) % 10 == 0 && lastTime != time(NULL))
                 {
                     lastTime = time(NULL);
-                    logger->info("Progress " + std::to_string(round(((float)progress / (float)filesize) * 1000.0f) / 10.0f) + "%%");
+                    logger->info("Progress " + std::to_string(round(((double)progress / (double)filesize) * 1000.0) / 10.0) + "%%");
                 }
             }
 
@@ -190,6 +198,7 @@ namespace eos
                 modis_products.instrument_name = "modis";
                 modis_products.has_timestamps = true;
                 modis_products.timestamp_type = satdump::ImageProducts::TIMESTAMP_IFOV;
+                modis_products.bit_depth = 12;
                 modis_products.set_tle(satellite_tle);
                 modis_products.set_timestamps(modis_reader.timestamps_250);
                 nlohmann::json proj_cfg;
@@ -203,7 +212,7 @@ namespace eos
 
                 for (int i = 0; i < 2; i++)
                 {
-                    image::Image<uint16_t> image = modis_reader.getImage250m(i);
+                    image::Image image = modis_reader.getImage250m(i);
                     // modis::modis_match_detector_histograms(image, 1 /*4*/, 40 * 2);
                     if (d_modis_bowtie)
                         image = image::bowtie::correctGenericBowTie(image, 1, scanHeight_250, alpha, beta);
@@ -212,21 +221,22 @@ namespace eos
 
                 for (int i = 0; i < 5; i++)
                 {
-                    image::Image<uint16_t> image = modis_reader.getImage500m(i);
+                    image::Image image = modis_reader.getImage500m(i);
                     // modis::modis_match_detector_histograms(image, 1 /*2*/, 20 * 2);
                     if (d_modis_bowtie)
                         image = image::bowtie::correctGenericBowTie(image, 1, scanHeight_500, alpha, beta);
                     modis_products.images.push_back({"MODIS-" + std::to_string(i + 3), std::to_string(i + 3), image});
                 }
 
+                std::vector<std::vector<int>> bowtie_lut_1km;
                 for (int i = 0; i < 31; i++)
                 {
-                    image::Image<uint16_t> image = modis_reader.getImage1000m(i);
+                    image::Image image = modis_reader.getImage1000m(i);
 
                     // modis::modis_match_detector_histograms(image, 1, 10 * 2);
 
                     if (d_modis_bowtie)
-                        image = image::bowtie::correctGenericBowTie(image, 1, scanHeight_1000, alpha, beta);
+                        image = image::bowtie::correctGenericBowTie(image, 1, scanHeight_1000, alpha, beta, &bowtie_lut_1km);
 
                     if (i < 5)
                         modis_products.images.push_back({"MODIS-" + std::to_string(i + 8), std::to_string(i + 8), image});
@@ -242,6 +252,36 @@ namespace eos
                         modis_products.images.push_back({"MODIS-" + std::to_string(i + 6), std::to_string(i + 6), image});
                 }
 
+                // Calibration
+                nlohmann::json calib_cfg;
+                calib_cfg["calibrator"] = "eos_modis";
+                calib_cfg["vars"] = modis::precompute::precomputeVars(&modis_products, modis_reader.getCalib(), d_satellite == AQUA);
+                calib_cfg["is_aqua"] = d_satellite == AQUA;
+                calib_cfg["bowtie_lut_1km"] = bowtie_lut_1km;
+                modis_products.set_calibration(calib_cfg);
+                for (int i = 0; i < 21; i++)
+                    modis_products.set_calibration_type(i, satdump::ImageProducts::CALIB_REFLECTANCE);
+                for (int i = 21; i < 38; i++)
+                    modis_products.set_calibration_type(i, satdump::ImageProducts::CALIB_RADIANCE);
+                modis_products.set_calibration_type(27, satdump::ImageProducts::CALIB_REFLECTANCE);
+
+                for (int i = 0; i < 38; i++)
+                { // Set to 0 for now
+                    modis_products.set_wavenumber(i, -1);
+                    modis_products.set_calibration_default_radiance_range(i, 0, 0);
+                }
+
+                auto modis_table = loadJsonFile(resources::getResourcePath("calibration/modis_table.json"));
+
+                for (int i = 0; i < 16; i++)
+                {
+                    int ch = i;
+                    if (ch >= 6)
+                        ch++;
+                    modis_products.set_wavenumber(21 + ch, freq_to_wavenumber(299792458.0 / modis_table["wavelengths"][i].get<double>()));
+                    modis_products.set_calibration_default_radiance_range(21 + ch, modis_table["default_ranges"][i][0].get<double>(), modis_table["default_ranges"][i][1].get<double>());
+                }
+
                 modis_products.save(directory);
                 dataset.products_list.push_back("MODIS");
 
@@ -339,14 +379,20 @@ namespace eos
                 logger->info("Lines (FM3) : " + std::to_string(ceres_fm3_reader.lines));
                 logger->info("Lines (FM4) : " + std::to_string(ceres_fm4_reader.lines));
 
-                WRITE_IMAGE(ceres_fm3_reader.getImage(0), directory + "/CERES1-SHORTWAVE");
-                WRITE_IMAGE(ceres_fm3_reader.getImage(1), directory + "/CERES1-LONGWAVE");
-                WRITE_IMAGE(ceres_fm3_reader.getImage(2), directory + "/CERES1-TOTAL");
+                auto img = ceres_fm3_reader.getImage(0);
+                image::save_img(img, directory + "/CERES1-SHORTWAVE");
+                img = ceres_fm3_reader.getImage(1);
+                image::save_img(img, directory + "/CERES1-LONGWAVE");
+                img = ceres_fm3_reader.getImage(2);
+                image::save_img(img, directory + "/CERES1-TOTAL");
                 ceres_fm3_status = DONE;
 
-                WRITE_IMAGE(ceres_fm4_reader.getImage(0), directory + "/CERES2-SHORTWAVE");
-                WRITE_IMAGE(ceres_fm4_reader.getImage(1), directory + "/CERES2-LONGWAVE");
-                WRITE_IMAGE(ceres_fm4_reader.getImage(2), directory + "/CERES2-TOTAL");
+                img = ceres_fm4_reader.getImage(0);
+                image::save_img(img, directory + "/CERES2-SHORTWAVE");
+                img = ceres_fm4_reader.getImage(1);
+                image::save_img(img, directory + "/CERES2-LONGWAVE");
+                img = ceres_fm4_reader.getImage(2);
+                image::save_img(img, directory + "/CERES2-TOTAL");
                 ceres_fm4_status = DONE;
             }
 
@@ -362,18 +408,22 @@ namespace eos
                 logger->info("Lines (UV) : " + std::to_string(omi_1_reader.lines));
                 logger->info("Lines (VIS) : " + std::to_string(omi_2_reader.lines));
 
-                WRITE_IMAGE(omi_1_reader.getImageRaw(), directory + "/OMI-1");
-                WRITE_IMAGE(omi_2_reader.getImageRaw(), directory + "/OMI-2");
+                auto img = omi_1_reader.getImageRaw();
+                image::save_img(img, directory + "/OMI-1");
+                img = omi_2_reader.getImageRaw();
+                image::save_img(img, directory + "/OMI-2");
 
-                WRITE_IMAGE(omi_1_reader.getImageVisible(), directory + "/OMI-VIS-1");
-                WRITE_IMAGE(omi_2_reader.getImageVisible(), directory + "/OMI-VIS-2");
+                img = omi_1_reader.getImageVisible();
+                image::save_img(img, directory + "/OMI-VIS-1");
+                img = omi_2_reader.getImageVisible();
+                image::save_img(img, directory + "/OMI-VIS-2");
 
-                image::Image<uint16_t> imageAll1 = image::make_manyimg_composite<uint16_t>(33, 24, 792, [this](int c)
-                                                                                           { return omi_1_reader.getChannel(c); });
-                image::Image<uint16_t> imageAll2 = image::make_manyimg_composite<uint16_t>(33, 24, 792, [this](int c)
-                                                                                           { return omi_2_reader.getChannel(c); });
-                WRITE_IMAGE(imageAll1, directory + "/OMI-ALL-1");
-                WRITE_IMAGE(imageAll2, directory + "/OMI-ALL-2");
+                image::Image imageAll1 = image::make_manyimg_composite(33, 24, 792, [this](int c)
+                                                                         { return omi_1_reader.getChannel(c); });
+                image::Image imageAll2 = image::make_manyimg_composite(33, 24, 792, [this](int c)
+                                                                         { return omi_2_reader.getChannel(c); });
+                image::save_img(imageAll1, directory + "/OMI-ALL-1");
+                image::save_img(imageAll2, directory + "/OMI-ALL-2");
                 omi_status = DONE;
             }
 
@@ -400,7 +450,7 @@ namespace eos
                     ImGui::TableSetColumnIndex(0);
                     ImGui::Text("MODIS");
                     ImGui::TableSetColumnIndex(1);
-                    ImGui::TextColored(ImColor(0, 255, 0), "%d", modis_reader.lines);
+                    ImGui::TextColored(style::theme.green, "%d", modis_reader.lines);
                     ImGui::TableSetColumnIndex(2);
                     drawStatus(modis_status);
                 }
@@ -411,7 +461,7 @@ namespace eos
                     ImGui::TableSetColumnIndex(0);
                     ImGui::Text("AIRS");
                     ImGui::TableSetColumnIndex(1);
-                    ImGui::TextColored(ImColor(0, 255, 0), "%d", airs_reader.lines);
+                    ImGui::TextColored(style::theme.green, "%d", airs_reader.lines);
                     ImGui::TableSetColumnIndex(2);
                     drawStatus(airs_status);
 
@@ -419,7 +469,7 @@ namespace eos
                     ImGui::TableSetColumnIndex(0);
                     ImGui::Text("AMSU A1");
                     ImGui::TableSetColumnIndex(1);
-                    ImGui::TextColored(ImColor(0, 255, 0), "%d", amsu_a1_reader.lines);
+                    ImGui::TextColored(style::theme.green, "%d", amsu_a1_reader.lines);
                     ImGui::TableSetColumnIndex(2);
                     drawStatus(amsu_status);
 
@@ -427,7 +477,7 @@ namespace eos
                     ImGui::TableSetColumnIndex(0);
                     ImGui::Text("AMSU A2");
                     ImGui::TableSetColumnIndex(1);
-                    ImGui::TextColored(ImColor(0, 255, 0), "%d", amsu_a2_reader.lines);
+                    ImGui::TextColored(style::theme.green, "%d", amsu_a2_reader.lines);
                     ImGui::TableSetColumnIndex(2);
                     drawStatus(amsu_status);
 
@@ -435,7 +485,7 @@ namespace eos
                     ImGui::TableSetColumnIndex(0);
                     ImGui::Text("CERES FM-3");
                     ImGui::TableSetColumnIndex(1);
-                    ImGui::TextColored(ImColor(0, 255, 0), "%d", ceres_fm3_reader.lines);
+                    ImGui::TextColored(style::theme.green, "%d", ceres_fm3_reader.lines);
                     ImGui::TableSetColumnIndex(2);
                     drawStatus(ceres_fm3_status);
 
@@ -443,7 +493,7 @@ namespace eos
                     ImGui::TableSetColumnIndex(0);
                     ImGui::Text("CERES FM-4");
                     ImGui::TableSetColumnIndex(1);
-                    ImGui::TextColored(ImColor(0, 255, 0), "%d", ceres_fm4_reader.lines);
+                    ImGui::TextColored(style::theme.green, "%d", ceres_fm4_reader.lines);
                     ImGui::TableSetColumnIndex(2);
                     drawStatus(ceres_fm4_status);
                 }
@@ -454,7 +504,7 @@ namespace eos
                     ImGui::TableSetColumnIndex(0);
                     ImGui::Text("OMI 1");
                     ImGui::TableSetColumnIndex(1);
-                    ImGui::TextColored(ImColor(0, 255, 0), "%d", omi_1_reader.lines);
+                    ImGui::TextColored(style::theme.green, "%d", omi_1_reader.lines);
                     ImGui::TableSetColumnIndex(2);
                     drawStatus(omi_status);
 
@@ -462,7 +512,7 @@ namespace eos
                     ImGui::TableSetColumnIndex(0);
                     ImGui::Text("OMI 2");
                     ImGui::TableSetColumnIndex(1);
-                    ImGui::TextColored(ImColor(0, 255, 0), "%d", omi_2_reader.lines);
+                    ImGui::TextColored(style::theme.green, "%d", omi_2_reader.lines);
                     ImGui::TableSetColumnIndex(2);
                     drawStatus(omi_status);
                 }
@@ -470,7 +520,7 @@ namespace eos
                 ImGui::EndTable();
             }
 
-            ImGui::ProgressBar((float)progress / (float)filesize, ImVec2(ImGui::GetWindowWidth() - 10, 20 * ui_scale));
+            ImGui::ProgressBar((double)progress / (double)filesize, ImVec2(ImGui::GetContentRegionAvail().x, 20 * ui_scale));
 
             ImGui::End();
         }
