@@ -14,6 +14,9 @@
 #include "resources.h"
 #include "instruments/mersi_histmatch.h"
 #include "instruments/mersi_offset_interleaved.h"
+#include "nlohmann/json_utils.h"
+#include "core/exception.h"
+#include "common/image/io.h"
 
 namespace fengyun3
 {
@@ -37,7 +40,7 @@ namespace fengyun3
             else if (parameters["satellite"] == "fy3g")
                 d_satellite = FY_3G;
             else
-                throw std::runtime_error("FY3 Instruments Decoder : FY3 satellite \"" + parameters["satellite"].get<std::string>() + "\" is not valid!");
+                throw satdump_exception("FY3 Instruments Decoder : FY3 satellite \"" + parameters["satellite"].get<std::string>() + "\" is not valid!");
 
             if (parameters["downlink"] == "ahrpt")
                 d_downlink = AHRPT;
@@ -390,10 +393,10 @@ namespace fengyun3
                     }
                     else if (vcdu.vcid == 12) // CCSDS-Compliant VCID
                     {
+#if 0
                         std::vector<ccsds::CCSDSPacket> ccsdsFrames = demuxer_vcid12.work(cadu);
                         for (ccsds::CCSDSPacket &pkt : ccsdsFrames)
                         {
-#if 0
                             printf("APID %d\n", pkt.header.apid);
                             if (pkt.header.apid == 1)
                             {
@@ -408,8 +411,8 @@ namespace fengyun3
                                     idk_out.write((char *)pkt.payload.data(), 862);
                                 }
                             }
-#endif
                         }
+#endif
                         //    if (pkt.header.apid == 16) // MWHS-2
                         //        mwhs2_reader.work(pkt, true);
                         //    else if (pkt.header.apid == 7) // MWTS-3
@@ -422,7 +425,7 @@ namespace fengyun3
                 if (time(NULL) % 10 == 0 && lastTime != time(NULL))
                 {
                     lastTime = time(NULL);
-                    logger->info("Progress " + std::to_string(round(((float)progress / (float)filesize) * 1000.0f) / 10.0f) + "%%");
+                    logger->info("Progress " + std::to_string(round(((double)progress / (double)filesize) * 1000.0) / 10.0) + "%%");
                 }
             }
 
@@ -431,7 +434,7 @@ namespace fengyun3
             if (d_dump_mersi)
                 mersi_bin.close();
 
-            int scid = most_common(fy_scids.begin(), fy_scids.end());
+            int scid = most_common(fy_scids.begin(), fy_scids.end(), 0);
             fy_scids.clear();
 
             std::string sat_name = "Unknown FengYun-3";
@@ -615,7 +618,7 @@ namespace fengyun3
 
                 virr_products.set_timestamps(virr_reader.timestamps);
                 if (d_downlink == AHRPT)
-                    dataset.timestamp = get_median(virr_reader.timestamps); //Re-set dataset timestamp since we just adjusted it
+                    dataset.timestamp = get_median(virr_reader.timestamps); // Re-set dataset timestamp since we just adjusted it
 
                 virr_products.save(directory);
                 dataset.products_list.push_back("VIRR");
@@ -674,7 +677,7 @@ namespace fengyun3
 
                 for (int i = 0; i < 20; i++)
                 {
-                    image::Image<uint16_t> image = mersi1_reader.getChannel(i);
+                    image::Image image = mersi1_reader.getChannel(i);
                     logger->debug("Processing channel %d", i + 1);
                     if (d_mersi_histmatch)
                         mersi::mersi_match_detector_histograms(image, i < 5 ? 40 : 10);
@@ -753,7 +756,7 @@ namespace fengyun3
 
                 for (int i = 0; i < 25; i++)
                 {
-                    image::Image<uint16_t> image = mersi2_reader.getChannel(i);
+                    image::Image image = mersi2_reader.getChannel(i);
                     logger->debug("Processing channel %d", i + 1);
                     if (d_mersi_histmatch)
                         mersi::mersi_match_detector_histograms(image, (i == 4 || i == 5) ? 80 : (i < 6 ? 40 : 10));
@@ -830,7 +833,7 @@ namespace fengyun3
 
                 for (int i = 0; i < 25; i++)
                 {
-                    image::Image<uint16_t> image = mersi3_reader.getChannel(i);
+                    image::Image image = mersi3_reader.getChannel(i);
                     logger->debug("Processing channel %d", i + 1);
                     if (d_mersi_histmatch)
                         mersi::mersi_match_detector_histograms(image, (i == 4 || i == 5) ? 80 : (i < 6 ? 40 : 10));
@@ -900,7 +903,7 @@ namespace fengyun3
 
                 for (int i = 0; i < 18; i++)
                 {
-                    image::Image<uint16_t> image = mersill_reader.getChannel(i);
+                    image::Image image = mersill_reader.getChannel(i);
                     logger->debug("Processing channel %d", i + 1);
                     if (d_mersi_histmatch)
                         mersi::mersi_match_detector_histograms(image, i < 2 ? 80 : 10);
@@ -947,21 +950,23 @@ namespace fengyun3
                 mersirm_products.set_timestamps(mersirm_reader.timestamps);
                 mersirm_products.set_proj_cfg(loadJsonFile(resources::getResourcePath("projections_settings/fengyun_g_mersirm.json")));
 
+                bool rotate = d_parameters.contains("satellite_rotated") ? d_parameters["satellite_rotated"].get<bool>() : false;
+
                 // Channel offsets relative to Ch1
                 int offset[8] = {
                     0,
+                    rotate ? -2 : 2,
+                    rotate ? -4 : 4,
+                    rotate ? -2 : 2,
                     0,
-                    0,
-                    0,
-                    0,
-                    0,
-                    0,
-                    0,
+                    rotate ? -7 : 7,
+                    rotate ? -2 : 2,
+                    rotate ? 2 : -1,
                 };
 
                 for (int i = 0; i < 8; i++)
                 {
-                    image::Image<uint16_t> image = mersirm_reader.getChannel(i);
+                    image::Image image = mersirm_reader.getChannel(i);
                     logger->debug("Processing channel %d", i + 1);
                     if (d_mersi_histmatch)
                         mersi::mersi_match_detector_histograms(image, 10);
@@ -969,7 +974,7 @@ namespace fengyun3
                         image = image::bowtie::correctGenericBowTie(image, 1, scanHeight_1000, alpha, beta);
 
                     // What did you do NSMC.... Why did you turn FY-3G 180 degs!?
-                    if (d_parameters.contains("satellite_rotated") ? d_parameters["satellite_rotated"].get<bool>() : false)
+                    if (rotate)
                     {
                         auto img2 = image;
                         for (int l = 0; l < (int)img2.height() / 10; l++)
@@ -978,7 +983,7 @@ namespace fengyun3
                             {
                                 for (int c = 0; c < 10; c++)
                                 {
-                                    image[(l * 10 + c) * img2.width() + x] = img2[(l * 10 + (9 - c)) * img2.width() + x];
+                                    image.set((l * 10 + c) * img2.width() + x, img2.get((l * 10 + (9 - c)) * img2.width() + x));
                                 }
                             }
                         }
@@ -1111,7 +1116,8 @@ namespace fengyun3
                 logger->info("----------- GAS");
                 logger->info("Lines : " + std::to_string(gas_reader.lines));
 
-                gas_reader.getChannel().save_img(directory + "/GAS");
+                auto img = gas_reader.getChannel();
+                image::save_img(img, directory + "/GAS");
 
                 gas_status = DONE;
             }
@@ -1261,7 +1267,7 @@ namespace fengyun3
                     ImGui::TableSetColumnIndex(0);
                     ImGui::Text("MERSI-1");
                     ImGui::TableSetColumnIndex(1);
-                    ImGui::TextColored(ImColor(0, 255, 0), "%d", mersi1_reader.segments);
+                    ImGui::TextColored(style::theme.green, "%d", mersi1_reader.segments);
                     ImGui::TableSetColumnIndex(2);
                     drawStatus(mersi1_status);
                 }
@@ -1272,7 +1278,7 @@ namespace fengyun3
                     ImGui::TableSetColumnIndex(0);
                     ImGui::Text("MERSI-2");
                     ImGui::TableSetColumnIndex(1);
-                    ImGui::TextColored(ImColor(0, 255, 0), "%d", mersi2_reader.segments);
+                    ImGui::TextColored(style::theme.green, "%d", mersi2_reader.segments);
                     ImGui::TableSetColumnIndex(2);
                     drawStatus(mersi2_status);
                 }
@@ -1283,7 +1289,7 @@ namespace fengyun3
                     ImGui::TableSetColumnIndex(0);
                     ImGui::Text("MERSI-LL");
                     ImGui::TableSetColumnIndex(1);
-                    ImGui::TextColored(ImColor(0, 255, 0), "%d", mersill_reader.segments);
+                    ImGui::TextColored(style::theme.green, "%d", mersill_reader.segments);
                     ImGui::TableSetColumnIndex(2);
                     drawStatus(mersill_status);
                 }
@@ -1294,7 +1300,7 @@ namespace fengyun3
                     ImGui::TableSetColumnIndex(0);
                     ImGui::Text("MERSI-3");
                     ImGui::TableSetColumnIndex(1);
-                    ImGui::TextColored(ImColor(0, 255, 0), "%d", mersi3_reader.segments);
+                    ImGui::TextColored(style::theme.green, "%d", mersi3_reader.segments);
                     ImGui::TableSetColumnIndex(2);
                     drawStatus(mersi3_status);
 
@@ -1302,7 +1308,7 @@ namespace fengyun3
                     ImGui::TableSetColumnIndex(0);
                     ImGui::Text("MWRI-2");
                     ImGui::TableSetColumnIndex(1);
-                    ImGui::TextColored(ImColor(0, 255, 0), "%d", mwri2_reader.lines);
+                    ImGui::TextColored(style::theme.green, "%d", mwri2_reader.lines);
                     ImGui::TableSetColumnIndex(2);
                     drawStatus(mwri2_status);
                 }
@@ -1313,7 +1319,7 @@ namespace fengyun3
                     ImGui::TableSetColumnIndex(0);
                     ImGui::Text("MERSI-RM");
                     ImGui::TableSetColumnIndex(1);
-                    ImGui::TextColored(ImColor(0, 255, 0), "%d", mersirm_reader.segments);
+                    ImGui::TextColored(style::theme.green, "%d", mersirm_reader.segments);
                     ImGui::TableSetColumnIndex(2);
                     drawStatus(mersirm_status);
 
@@ -1321,7 +1327,7 @@ namespace fengyun3
                     ImGui::TableSetColumnIndex(0);
                     ImGui::Text("MWRI-RM");
                     ImGui::TableSetColumnIndex(1);
-                    ImGui::TextColored(ImColor(0, 255, 0), "%d", mwrirm_reader.lines);
+                    ImGui::TextColored(style::theme.green, "%d", mwrirm_reader.lines);
                     ImGui::TableSetColumnIndex(2);
                     drawStatus(mwrirm_status);
 
@@ -1329,7 +1335,7 @@ namespace fengyun3
                     ImGui::TableSetColumnIndex(0);
                     ImGui::Text("PMR-1");
                     ImGui::TableSetColumnIndex(1);
-                    ImGui::TextColored(ImColor(0, 255, 0), "%d", pmr1_reader->images_count);
+                    ImGui::TextColored(style::theme.green, "%d", pmr1_reader->images_count);
                     ImGui::TableSetColumnIndex(2);
                     drawStatus(pmr1_status);
 
@@ -1337,7 +1343,7 @@ namespace fengyun3
                     ImGui::TableSetColumnIndex(0);
                     ImGui::Text("PMR-2");
                     ImGui::TableSetColumnIndex(1);
-                    ImGui::TextColored(ImColor(0, 255, 0), "%d", pmr2_reader->images_count);
+                    ImGui::TextColored(style::theme.green, "%d", pmr2_reader->images_count);
                     ImGui::TableSetColumnIndex(2);
                     drawStatus(pmr2_status);
                 }
@@ -1348,7 +1354,7 @@ namespace fengyun3
                     ImGui::TableSetColumnIndex(0);
                     ImGui::Text("VIRR");
                     ImGui::TableSetColumnIndex(1);
-                    ImGui::TextColored(ImColor(0, 255, 0), "%d", virr_reader.lines);
+                    ImGui::TextColored(style::theme.green, "%d", virr_reader.lines);
                     ImGui::TableSetColumnIndex(2);
                     drawStatus(virr_status);
 
@@ -1356,7 +1362,7 @@ namespace fengyun3
                     ImGui::TableSetColumnIndex(0);
                     ImGui::Text("ERM");
                     ImGui::TableSetColumnIndex(1);
-                    ImGui::TextColored(ImColor(0, 255, 0), "%d", erm_reader.lines);
+                    ImGui::TextColored(style::theme.green, "%d", erm_reader.lines);
                     ImGui::TableSetColumnIndex(2);
                     drawStatus(erm_status);
                 }
@@ -1367,7 +1373,7 @@ namespace fengyun3
                     ImGui::TableSetColumnIndex(0);
                     ImGui::Text("MWTS-1");
                     ImGui::TableSetColumnIndex(1);
-                    ImGui::TextColored(ImColor(0, 255, 0), "%d", mwts1_reader.lines);
+                    ImGui::TextColored(style::theme.green, "%d", mwts1_reader.lines);
                     ImGui::TableSetColumnIndex(2);
                     drawStatus(mwts1_status);
                 }
@@ -1378,7 +1384,7 @@ namespace fengyun3
                     ImGui::TableSetColumnIndex(0);
                     ImGui::Text("MWHS-1");
                     ImGui::TableSetColumnIndex(1);
-                    ImGui::TextColored(ImColor(0, 255, 0), "%d", mwhs1_reader.lines);
+                    ImGui::TextColored(style::theme.green, "%d", mwhs1_reader.lines);
                     ImGui::TableSetColumnIndex(2);
                     drawStatus(mwhs1_status);
                 }
@@ -1389,7 +1395,7 @@ namespace fengyun3
                     ImGui::TableSetColumnIndex(0);
                     ImGui::Text("MWTS-2");
                     ImGui::TableSetColumnIndex(1);
-                    ImGui::TextColored(ImColor(0, 255, 0), "%d", mwts2_reader.lines);
+                    ImGui::TextColored(style::theme.green, "%d", mwts2_reader.lines);
                     ImGui::TableSetColumnIndex(2);
                     drawStatus(mwts2_status);
                 }
@@ -1400,7 +1406,7 @@ namespace fengyun3
                     ImGui::TableSetColumnIndex(0);
                     ImGui::Text("MWRI");
                     ImGui::TableSetColumnIndex(1);
-                    ImGui::TextColored(ImColor(0, 255, 0), "%d", mwri_reader.lines);
+                    ImGui::TextColored(style::theme.green, "%d", mwri_reader.lines);
                     ImGui::TableSetColumnIndex(2);
                     drawStatus(mwri_status);
                 }
@@ -1411,7 +1417,7 @@ namespace fengyun3
                     ImGui::TableSetColumnIndex(0);
                     ImGui::Text("WAI");
                     ImGui::TableSetColumnIndex(1);
-                    ImGui::TextColored(ImColor(0, 255, 0), "%d", wai_reader->images_count);
+                    ImGui::TextColored(style::theme.green, "%d", wai_reader->images_count);
                     ImGui::TableSetColumnIndex(2);
                     drawStatus(wai_status);
                 }
@@ -1422,7 +1428,7 @@ namespace fengyun3
                     ImGui::TableSetColumnIndex(0);
                     ImGui::Text("GAS");
                     ImGui::TableSetColumnIndex(1);
-                    ImGui::TextColored(ImColor(0, 255, 0), "%d", gas_reader.lines);
+                    ImGui::TextColored(style::theme.green, "%d", gas_reader.lines);
                     ImGui::TableSetColumnIndex(2);
                     drawStatus(gas_status);
                 }
@@ -1433,7 +1439,7 @@ namespace fengyun3
                     ImGui::TableSetColumnIndex(0);
                     ImGui::Text("MWTS-3");
                     ImGui::TableSetColumnIndex(1);
-                    ImGui::TextColored(ImColor(0, 255, 0), "%d", mwts3_reader.lines);
+                    ImGui::TextColored(style::theme.green, "%d", mwts3_reader.lines);
                     ImGui::TableSetColumnIndex(2);
                     drawStatus(mwts3_status);
                 }
@@ -1444,7 +1450,7 @@ namespace fengyun3
                     ImGui::TableSetColumnIndex(0);
                     ImGui::Text("MWHS-2");
                     ImGui::TableSetColumnIndex(1);
-                    ImGui::TextColored(ImColor(0, 255, 0), "%d", mwhs2_reader.lines);
+                    ImGui::TextColored(style::theme.green, "%d", mwhs2_reader.lines);
                     ImGui::TableSetColumnIndex(2);
                     drawStatus(mwhs2_status);
                 }
@@ -1455,7 +1461,7 @@ namespace fengyun3
                     ImGui::TableSetColumnIndex(0);
                     ImGui::Text("XEUVI");
                     ImGui::TableSetColumnIndex(1);
-                    ImGui::TextColored(ImColor(0, 255, 0), "%d", xeuvi_reader->images_count);
+                    ImGui::TextColored(style::theme.green, "%d", xeuvi_reader->images_count);
                     ImGui::TableSetColumnIndex(2);
                     drawStatus(xeuvi_status);
 
@@ -1463,7 +1469,7 @@ namespace fengyun3
                     ImGui::TableSetColumnIndex(0);
                     ImGui::Text("WindRAD C-Band");
                     ImGui::TableSetColumnIndex(1);
-                    ImGui::TextColored(ImColor(0, 255, 0), "%d", windrad_reader1->imgCount);
+                    ImGui::TextColored(style::theme.green, "%d", windrad_reader1->imgCount);
                     ImGui::TableSetColumnIndex(2);
                     drawStatus(windrad_C_status);
 
@@ -1471,7 +1477,7 @@ namespace fengyun3
                     ImGui::TableSetColumnIndex(0);
                     ImGui::Text("WindRAD Ku-Band");
                     ImGui::TableSetColumnIndex(1);
-                    ImGui::TextColored(ImColor(0, 255, 0), "%d", windrad_reader2->imgCount);
+                    ImGui::TextColored(style::theme.green, "%d", windrad_reader2->imgCount);
                     ImGui::TableSetColumnIndex(2);
                     drawStatus(windrad_Ku_status);
                 }
@@ -1479,7 +1485,7 @@ namespace fengyun3
                 ImGui::EndTable();
             }
 
-            ImGui::ProgressBar((float)progress / (float)filesize, ImVec2(ImGui::GetWindowWidth() - 10, 20 * ui_scale));
+            ImGui::ProgressBar((double)progress / (double)filesize, ImVec2(ImGui::GetContentRegionAvail().x, 20 * ui_scale));
 
             ImGui::End();
         }
