@@ -5,6 +5,7 @@
 #include "libs/miniz/miniz_zip.h"
 #include "imgui/imgui_image.h"
 #include <filesystem>
+#include "common/image/io.h"
 
 #ifdef _MSC_VER
 #define timegm _mkgmtime
@@ -14,18 +15,6 @@ namespace goes
 {
     namespace hrit
     {
-        std::string getHRITImageFilename(std::tm *timeReadable, std::string sat_name, int channel)
-        {
-            std::string utc_filename = sat_name + "_" + std::to_string(channel) + "_" +                                                                             // Satellite name and channel
-                                       std::to_string(timeReadable->tm_year + 1900) +                                                                               // Year yyyy
-                                       (timeReadable->tm_mon + 1 > 9 ? std::to_string(timeReadable->tm_mon + 1) : "0" + std::to_string(timeReadable->tm_mon + 1)) + // Month MM
-                                       (timeReadable->tm_mday > 9 ? std::to_string(timeReadable->tm_mday) : "0" + std::to_string(timeReadable->tm_mday)) + "T" +    // Day dd
-                                       (timeReadable->tm_hour > 9 ? std::to_string(timeReadable->tm_hour) : "0" + std::to_string(timeReadable->tm_hour)) +          // Hour HH
-                                       (timeReadable->tm_min > 9 ? std::to_string(timeReadable->tm_min) : "0" + std::to_string(timeReadable->tm_min)) +             // Minutes mm
-                                       (timeReadable->tm_sec > 9 ? std::to_string(timeReadable->tm_sec) : "0" + std::to_string(timeReadable->tm_sec)) + "Z";        // Seconds ss
-            return utc_filename;
-        }
-
         std::string getHRITImageFilename(std::tm *timeReadable, std::string sat_name, std::string channel)
         {
             std::string utc_filename = sat_name + "_" + channel + "_" +                                                                                             // Satellite name and channel
@@ -45,7 +34,7 @@ namespace goes
             if (!std::filesystem::exists(path))
                 std::filesystem::create_directory(path);
 
-            //If segmented, rename file after segment name
+            // If segmented, rename file after segment name
             std::string current_filename = file.filename;
             if (file.hasHeader<SegmentIdentificationHeader>())
             {
@@ -63,6 +52,41 @@ namespace goes
             fileo.close();
         }
 
+        void GOESLRITDataDecoderModule::saveImageP(GOESxRITProductMeta meta, image::Image &img)
+        {
+            if (meta.is_goesn)
+                img.resize(img.width(), img.height() * 1.75);
+
+            if (meta.channel == -1 || meta.satellite_name == "" || meta.satellite_short_name == "" || meta.scan_time == 0)
+            {
+                std::string ext;
+                image::append_ext(&ext, true);
+                if (std::filesystem::exists(directory + "/IMAGES/Unknown/" + meta.filename + ext))
+                {
+                    int current_iteration = 1;
+                    std::string filename_new;
+                    do
+                    {
+                        filename_new = meta.filename + "_" + std::to_string(current_iteration++);
+                    } while (std::filesystem::exists(directory + "/IMAGES/Unknown/" + filename_new + ext));
+                    image::save_img(img, directory + "/IMAGES/Unknown/" + filename_new);
+                    logger->warn("Image already existed. Written as %s", filename_new.c_str());
+                }
+                else
+                    image::save_img(img, directory + "/IMAGES/Unknown/" + meta.filename);
+            }
+            else
+            {
+                if (meta.satellite_name == "Himawari")
+                    productizer.setInstrumentID("ahi");
+                else if (meta.is_goesn)
+                    productizer.setInstrumentID("goesn_imager");
+                productizer.saveImage(img, 8 /*bit depth on GOES is ALWAYS 8*/, directory + "/IMAGES", meta.satellite_name, meta.satellite_short_name, std::to_string(meta.channel), meta.scan_time, meta.region, meta.image_navigation_record.get(), meta.image_data_function_record.get());
+                if (meta.satellite_name == "Himawari" || meta.is_goesn)
+                    productizer.setInstrumentID("abi");
+            }
+        }
+
         void GOESLRITDataDecoderModule::processLRITFile(::lrit::LRITFile &file)
         {
             std::string current_filename = file.filename;
@@ -70,9 +94,19 @@ namespace goes
             ::lrit::PrimaryHeader primary_header = file.getHeader<::lrit::PrimaryHeader>();
             NOAALRITHeader noaa_header = file.getHeader<NOAALRITHeader>();
 
-            // Check if this is image data, and if so also write it as an image
-            if (write_images && primary_header.file_type_code == 0 && file.hasHeader<::lrit::ImageStructureRecord>())
+            // Handle LRIT files with no data
+            if (primary_header.total_header_length == file.lrit_data.size())
             {
+                logger->warn("Received LRIT header with no body! Saving as .lrit");
+                saveLRITFile(file, directory + "/LRIT");
+            }
+
+            // Check if this is image data, and if so also write it as an image
+            else if (primary_header.file_type_code == 0 && file.hasHeader<::lrit::ImageStructureRecord>())
+            {
+                if (!write_images)
+                    return;
+
                 if (!std::filesystem::exists(directory + "/IMAGES"))
                     std::filesystem::create_directory(directory + "/IMAGES");
 
@@ -82,8 +116,17 @@ namespace goes
                 std::tm *timeReadable = gmtime(&timestamp_record.timestamp);
 
                 std::string old_filename = current_filename;
+                bool used_ancillary_proj = false;
+                GOESxRITProductMeta lmeta;
+                lmeta.filename = file.filename;
 
-                bool is_goesn = false;
+                // Try to parse navigation
+                if (file.hasHeader<::lrit::ImageNavigationRecord>())
+                    lmeta.image_navigation_record = std::make_shared<::lrit::ImageNavigationRecord>(file.getHeader<::lrit::ImageNavigationRecord>());
+
+                // Try to parse calibration
+                if (file.hasHeader<::lrit::ImageDataFunctionRecord>())
+                    lmeta.image_data_function_record = std::make_shared<::lrit::ImageDataFunctionRecord>(file.getHeader<::lrit::ImageDataFunctionRecord>());
 
                 // Process as a specific dataset
                 {
@@ -94,18 +137,45 @@ namespace goes
                                                                noaa_header.product_id == 18 ||
                                                                noaa_header.product_id == 19))
                     {
+                        lmeta.satellite_name = "GOES-" + std::to_string(noaa_header.product_id);
+                        lmeta.satellite_short_name = "G" + std::to_string(noaa_header.product_id);
                         std::vector<std::string> cutFilename = splitString(current_filename, '-');
 
                         if (cutFilename.size() >= 4)
                         {
                             int mode = -1;
-                            int channel = -1;
 
-                            if (sscanf(cutFilename[3].c_str(), "M%dC%02d", &mode, &channel) == 2)
+                            if (sscanf(cutFilename[3].c_str(), "M%dC%02d", &mode, &lmeta.channel) == 2)
                             {
                                 AncillaryTextRecord ancillary_record = file.getHeader<AncillaryTextRecord>();
 
-                                std::string region = "Others";
+                                // On GOES-R HRIT, the projection information in the Image Navigation Header is not accurate enough. Use the data
+                                // in the Ancillary record instead
+                                if (ancillary_record.meta.count("perspective_point_height") > 0 &&
+                                    ancillary_record.meta.count("y_add_offset") > 0 && ancillary_record.meta.count("y_scale_factor") > 0)
+                                {
+                                    used_ancillary_proj = true;
+                                    double scale_factor = std::stod(ancillary_record.meta["y_scale_factor"]);
+                                    lmeta.image_navigation_record->line_scalar =
+                                        std::abs(std::stod(ancillary_record.meta["perspective_point_height"]) * scale_factor);
+                                    lmeta.image_navigation_record->line_offset = -std::stod(ancillary_record.meta["y_add_offset"]) / scale_factor;
+
+                                    // Avoid upstream rounding errors on smaller images
+                                    if (image_structure_record.columns_count < 5424)
+                                        lmeta.image_navigation_record->line_offset = std::ceil(lmeta.image_navigation_record->line_offset);
+                                }
+                                if (ancillary_record.meta.count("perspective_point_height") > 0 &&
+                                    ancillary_record.meta.count("x_add_offset") > 0 && ancillary_record.meta.count("x_scale_factor") > 0)
+                                {
+                                    double scale_factor = std::stod(ancillary_record.meta["x_scale_factor"]);
+                                    lmeta.image_navigation_record->column_scalar =
+                                        std::abs(std::stod(ancillary_record.meta["perspective_point_height"]) * scale_factor);
+                                    lmeta.image_navigation_record->column_offset = -std::stod(ancillary_record.meta["x_add_offset"]) / scale_factor;
+
+                                    // Avoid upstream rounding errors on smaller images
+                                    if (image_structure_record.columns_count < 5424)
+                                        lmeta.image_navigation_record->column_offset = std::ceil(lmeta.image_navigation_record->column_offset);
+                                }
 
                                 // Parse Region
                                 if (ancillary_record.meta.count("Region") > 0)
@@ -114,16 +184,16 @@ namespace goes
 
                                     if (regionName == "Full Disk")
                                     {
-                                        region = "Full Disk";
+                                        lmeta.region = "Full Disk";
                                     }
                                     else if (regionName == "Mesoscale")
                                     {
                                         if (cutFilename[2] == "CMIPM1")
-                                            region = "Mesoscale 1";
+                                            lmeta.region = "Mesoscale 1";
                                         else if (cutFilename[2] == "CMIPM2")
-                                            region = "Mesoscale 2";
+                                            lmeta.region = "Mesoscale 2";
                                         else
-                                            region = "Mesoscale";
+                                            lmeta.region = "Mesoscale";
                                     }
                                 }
 
@@ -133,39 +203,7 @@ namespace goes
                                 {
                                     std::string scanTime = ancillary_record.meta["Time of frame start"];
                                     strptime(scanTime.c_str(), "%Y-%m-%dT%H:%M:%S", &scanTimestamp);
-                                }
-
-                                std::string subdir = "GOES-" + std::to_string(noaa_header.product_id) + "/" + region;
-
-                                if (!std::filesystem::exists(directory + "/IMAGES/" + subdir))
-                                    std::filesystem::create_directories(directory + "/IMAGES/" + subdir);
-
-                                current_filename = subdir + "/" + getHRITImageFilename(&scanTimestamp, "G" + std::to_string(noaa_header.product_id), channel);
-
-                                //Configure mesoscale color compoer - FD is defined further down
-                                if ((region == "Mesoscale 1" || region == "Mesoscale 2"))
-                                {
-                                    std::shared_ptr<GOESRFalseColorComposer> goes_r_fc_composer;
-
-                                    if (region == "Mesoscale 1")
-                                        goes_r_fc_composer = goes_r_fc_composer_meso1;
-                                    else if (region == "Mesoscale 2")
-                                        goes_r_fc_composer = goes_r_fc_composer_meso2;
-
-                                    image::Image<uint8_t> image(&file.lrit_data[primary_header.total_header_length],
-                                                                image_structure_record.columns_count,
-                                                                image_structure_record.lines_count, 1);
-
-                                    if (channel == 2)
-                                    {
-                                        goes_r_fc_composer->push2(image, timegm(&scanTimestamp));
-                                    }
-                                    else if (channel == 13)
-                                    {
-                                        goes_r_fc_composer->push13(image, timegm(&scanTimestamp));
-                                    }
-
-                                    goes_r_fc_composer->filename = subdir + "/" + getHRITImageFilename(&scanTimestamp, "G" + std::to_string(noaa_header.product_id), "FC");
+                                    lmeta.scan_time = mktime_utc(&scanTimestamp);
                                 }
                             }
                         }
@@ -175,36 +213,34 @@ namespace goes
                                                                     noaa_header.product_id == 14 ||
                                                                     noaa_header.product_id == 15))
                     {
-                        is_goesn = true;
-
-                        int channel = -1;
+                        lmeta.satellite_name = "GOES-" + std::to_string(noaa_header.product_id);
+                        lmeta.satellite_short_name = "G" + std::to_string(noaa_header.product_id);
+                        lmeta.is_goesn = true;
 
                         // Parse channel
                         if (noaa_header.product_subid <= 10)
-                            channel = 4;
+                            lmeta.channel = 4;
                         else if (noaa_header.product_subid <= 20)
-                            channel = 1;
+                            lmeta.channel = 1;
                         else if (noaa_header.product_subid <= 30)
-                            channel = 3;
-
-                        std::string region = "Others";
+                            lmeta.channel = 3;
 
                         // Parse Region. Had to peak in goestools again...
                         if (noaa_header.product_subid % 10 == 1)
-                            region = "Full Disk";
+                            lmeta.region = "Full Disk";
                         else if (noaa_header.product_subid % 10 == 2)
-                            region = "Northern Hemisphere";
+                            lmeta.region = "Northern Hemisphere";
                         else if (noaa_header.product_subid % 10 == 3)
-                            region = "Southern Hemisphere";
+                            lmeta.region = "Southern Hemisphere";
                         else if (noaa_header.product_subid % 10 == 4)
-                            region = "United States";
+                            lmeta.region = "United States";
                         else
                         {
                             char buf[32];
                             size_t len;
                             int num = (noaa_header.product_subid % 10) - 5;
                             len = snprintf(buf, 32, "Special Interest %d", num);
-                            region = std::string(buf, len);
+                            lmeta.region = std::string(buf, len);
                         }
 
                         // Parse scan time
@@ -214,22 +250,24 @@ namespace goes
                         {
                             std::string scanTime = ancillary_record.meta["Time of frame start"];
                             strptime(scanTime.c_str(), "%Y-%m-%dT%H:%M:%S", &scanTimestamp);
+                            lmeta.scan_time = mktime_utc(&scanTimestamp);
                         }
-
-                        std::string subdir = "GOES-" + std::to_string(noaa_header.product_id) + "/" + region;
-
-                        if (!std::filesystem::exists(directory + "/IMAGES/" + subdir))
-                            std::filesystem::create_directories(directory + "/IMAGES/" + subdir);
-
-                        current_filename = subdir + "/" + getHRITImageFilename(&scanTimestamp, "G" + std::to_string(noaa_header.product_id), channel);
                     }
-                    // Himawari-8 rebroadcast
-                    else if (primary_header.file_type_code == 0 && noaa_header.product_id == 43)
+                    // Himawari rebroadcast
+                    else if (primary_header.file_type_code == 0 && noaa_header.product_id == ID_HIMAWARI)
                     {
-                        std::string subdir = "Himawari-8/Full Disk";
+                        lmeta.satellite_name = "Himawari";
+                        lmeta.satellite_short_name = "HIM";
+                        lmeta.channel = noaa_header.product_subid;
+                        lmeta.region = "";
 
-                        if (!std::filesystem::exists(directory + "/IMAGES/" + subdir))
-                            std::filesystem::create_directories(directory + "/IMAGES/" + subdir);
+                        // Translate to real numbers
+                        if (lmeta.channel == 3)
+                            lmeta.channel = 13;
+                        else if (lmeta.channel == 7)
+                            lmeta.channel = 8;
+                        else if (lmeta.channel == 1)
+                            lmeta.channel = 3;
 
                         // Apparently the timestamp is in there for Himawari-8 data
                         AnnotationRecord annotation_record = file.getHeader<AnnotationRecord>();
@@ -238,7 +276,7 @@ namespace goes
                         if (strParts.size() > 3)
                         {
                             strptime(strParts[2].c_str(), "%Y%m%d%H%M", timeReadable);
-                            current_filename = subdir + "/" + getHRITImageFilename(timeReadable, "HIM", noaa_header.product_subid); // SubID = Channel
+                            lmeta.scan_time = mktime_utc(timeReadable);
                         }
                     }
                     // NWS Images
@@ -258,82 +296,55 @@ namespace goes
                 {
                     SegmentIdentificationHeader segment_id_header = file.getHeader<SegmentIdentificationHeader>();
 
-                    if (all_wip_images.count(file.vcid) == 0)
-                        all_wip_images.insert({file.vcid, std::make_unique<wip_images>()});
+                    // Internal "VCID" to use. Due to interleaving of channel segments on the Himawari relay,
+                    // we need to use adjusted VCIDs to keep one channel's segment from prematurely ending
+                    // another in-progress channel. Use VCID 70+ as these channels should not be used OTA.
+                    int vcid = file.vcid;
+                    if (noaa_header.product_id == ID_HIMAWARI)
+                        vcid = 70 + lmeta.channel;
 
-                    std::unique_ptr<wip_images> &wip_img = all_wip_images[file.vcid];
+                    if (all_wip_images.count(vcid) == 0)
+                        all_wip_images.insert({vcid, std::make_unique<wip_images>()});
+
+                    std::unique_ptr<wip_images> &wip_img = all_wip_images[vcid];
 
                     wip_img->imageStatus = RECEIVING;
 
-                    if (segmentedDecoders.count(file.vcid) == 0)
-                        segmentedDecoders.insert({file.vcid, SegmentedLRITImageDecoder()});
+                    if (segmentedDecoders.count(vcid) == 0)
+                        segmentedDecoders.insert({vcid, SegmentedLRITImageDecoder()});
 
-                    SegmentedLRITImageDecoder &segmentedDecoder = segmentedDecoders[file.vcid];
+                    SegmentedLRITImageDecoder &segmentedDecoder = segmentedDecoders[vcid];
 
-                    if (segmentedDecoder.image_id != segment_id_header.image_identifier)
+                    if (lmeta.image_navigation_record && noaa_header.product_id != ID_HIMAWARI && !used_ancillary_proj)
+                        lmeta.image_navigation_record->line_offset = lmeta.image_navigation_record->line_offset +
+                                                                     segment_id_header.segment_sequence_number * (segment_id_header.max_row / segment_id_header.max_segment);
+
+                    uint16_t image_identifier = segment_id_header.image_identifier;
+                    if (noaa_header.product_id == ID_HIMAWARI) // Image IDs are invalid for Himawari; make one up
+                        image_identifier = lmeta.scan_time % 10000 + lmeta.channel;
+
+                    if (segmentedDecoder.image_id != image_identifier)
                     {
                         if (segmentedDecoder.image_id != -1)
                         {
                             wip_img->imageStatus = SAVING;
-                            if (is_goesn)
-                                segmentedDecoder.image.resize(segmentedDecoder.image.width(), segmentedDecoder.image.height() * 1.75);
-                            segmentedDecoder.image.save_img(std::string(directory + "/IMAGES/" + segmentedDecoder.filename).c_str());
+                            saveImageP(segmentedDecoder.meta, *segmentedDecoder.image);
                             wip_img->imageStatus = RECEIVING;
                         }
 
                         segmentedDecoder = SegmentedLRITImageDecoder(segment_id_header.max_segment,
-                                                                     image_structure_record.columns_count,
-                                                                     image_structure_record.lines_count,
-                                                                     segment_id_header.image_identifier);
-                        segmentedDecoder.filename = current_filename;
+                                                                     segment_id_header.max_column,
+                                                                     segment_id_header.max_row,
+                                                                     image_identifier);
+                        segmentedDecoder.meta = lmeta;
                     }
 
                     if (noaa_header.product_id == ID_HIMAWARI)
-                        segmentedDecoder.pushSegment(&file.lrit_data[primary_header.total_header_length], segment_id_header.segment_sequence_number - 1);
+                        segmentedDecoder.pushSegment(&file.lrit_data[primary_header.total_header_length],
+                                                     file.lrit_data.size() - primary_header.total_header_length, segment_id_header.segment_sequence_number - 1);
                     else
-                        segmentedDecoder.pushSegment(&file.lrit_data[primary_header.total_header_length], segment_id_header.segment_sequence_number);
-
-                    // Check if this is GOES-R, if yes, this is Full Disk
-                    if (primary_header.file_type_code == 0 && (noaa_header.product_id == 16 ||
-                                                               noaa_header.product_id == 17 ||
-                                                               noaa_header.product_id == 18 ||
-                                                               noaa_header.product_id == 19))
-                    {
-                        int mode = -1;
-                        int channel = -1;
-                        std::vector<std::string> cutFilename = splitString(old_filename, '-');
-                        if (cutFilename.size() > 3)
-                        {
-                            if (sscanf(cutFilename[3].c_str(), "M%dC%02d", &mode, &channel) == 2)
-                            {
-                                AncillaryTextRecord ancillary_record = file.getHeader<AncillaryTextRecord>();
-
-                                // Parse scan time
-                                std::tm scanTimestamp = *timeReadable;                      // Default to CCSDS timestamp normally...
-                                if (ancillary_record.meta.count("Time of frame start") > 0) // ...unless we have a proper scan time
-                                {
-                                    std::string scanTime = ancillary_record.meta["Time of frame start"];
-                                    strptime(scanTime.c_str(), "%Y-%m-%dT%H:%M:%S", &scanTimestamp);
-                                }
-
-                                if (channel == 2)
-                                {
-                                    goes_r_fc_composer_full_disk->push2(segmentedDecoder.image, timegm(&scanTimestamp));
-                                    std::string subdir = "GOES-" + std::to_string(noaa_header.product_id) + "/Full Disk";
-                                    goes_r_fc_composer_full_disk->filename = subdir + "/" + getHRITImageFilename(&scanTimestamp, "G" + std::to_string(noaa_header.product_id), "FC");
-                                }
-
-                                else if (channel == 13 && file.vcid == 13) //Redundant check keeps relayed channel 13 from entering the color composer
-                                {
-                                    goes_r_fc_composer_full_disk->push13(segmentedDecoder.image, timegm(&scanTimestamp));
-                                    std::string subdir = "GOES-" + std::to_string(noaa_header.product_id) + "/Full Disk";
-                                    goes_r_fc_composer_full_disk->filename = subdir + "/" + getHRITImageFilename(&scanTimestamp, "G" + std::to_string(noaa_header.product_id), "FC");
-                                }
-
-
-                            }
-                        }
-                    }
+                        segmentedDecoder.pushSegment(&file.lrit_data[primary_header.total_header_length],
+                                                     file.lrit_data.size() - primary_header.total_header_length, segment_id_header.segment_sequence_number);
 
                     // If the UI is active, update texture
                     if (wip_img->textureID > 0)
@@ -341,39 +352,19 @@ namespace goes
                         // Downscale image
                         wip_img->img_height = 1000;
                         wip_img->img_width = 1000;
-                        image::Image<uint8_t> imageScaled = segmentedDecoder.image;
+                        image::Image imageScaled = *(segmentedDecoder.image);
                         imageScaled.resize(wip_img->img_width, wip_img->img_height);
-                        uchar_to_rgba(imageScaled.data(), wip_img->textureBuffer, wip_img->img_height * wip_img->img_width);
+                        if (imageScaled.typesize() == 1)
+                            image::image_to_rgba(imageScaled, wip_img->textureBuffer);
                         wip_img->hasToUpdate = true;
                     }
 
                     if (segmentedDecoder.isComplete())
                     {
                         wip_img->imageStatus = SAVING;
-                        if (is_goesn)
-                            segmentedDecoder.image.resize(segmentedDecoder.image.width(), segmentedDecoder.image.height() * 1.75);
-                        segmentedDecoder.image.save_img(std::string(directory + "/IMAGES/" + current_filename).c_str());
+                        saveImageP(segmentedDecoder.meta, *segmentedDecoder.image);
                         segmentedDecoder = SegmentedLRITImageDecoder();
                         wip_img->imageStatus = IDLE;
-
-                        // Check if this is GOES-R
-                        if (primary_header.file_type_code == 0 && (noaa_header.product_id == 16 ||
-                                                                   noaa_header.product_id == 17 ||
-                                                                   noaa_header.product_id == 18 ||
-                                                                   noaa_header.product_id == 19))
-                        {
-                            int mode = -1;
-                            int channel = -1;
-                            std::vector<std::string> cutFilename = splitString(old_filename, '-');
-                            if (cutFilename.size() > 3)
-                            {
-                                if (sscanf(cutFilename[3].c_str(), "M%dC%02d", &mode, &channel) == 2)
-                                {
-                                    if (goes_r_fc_composer_full_disk->hasData && channel == 2)
-                                        goes_r_fc_composer_full_disk->save();
-                                }
-                            }
-                        }
                     }
                 }
                 else
@@ -391,45 +382,19 @@ namespace goes
                     }
                     else // Write raw image dats
                     {
-                        //Sometimes, multiple different images can be sent down with the same name
-                        //Do not overwrite files
-                        std::string suffix = "";
-                        std::string extension = "";
-                        int suffixInt = 1;
-
-                        image::Image<uint8_t> image(&file.lrit_data[primary_header.total_header_length], image_structure_record.columns_count, image_structure_record.lines_count, 1);
-                        image.append_ext(&extension);
-
-                        while(std::filesystem::exists(directory + "/IMAGES/" + current_filename + suffix + extension))
-                        {
-                            suffixInt++;
-                            suffix = "-" + std::to_string(suffixInt);
-                        }
-
-                        if (is_goesn)
-                            image.resize(image.width(), image.height() * 1.75);
-                        image.save_img(std::string(directory + "/IMAGES/" + current_filename + suffix).c_str());
-
-                        // Check if this is GOES-R
-                        if (primary_header.file_type_code == 0 && (noaa_header.product_id == 16 ||
-                                                                   noaa_header.product_id == 17 ||
-                                                                   noaa_header.product_id == 18 ||
-                                                                   noaa_header.product_id == 19))
-                        {
-                            if (goes_r_fc_composer_meso1->hasData)
-                                goes_r_fc_composer_meso1->save();
-                            if (goes_r_fc_composer_meso2->hasData)
-                                goes_r_fc_composer_meso2->save();
-                        }
+                        image::Image image(&file.lrit_data[primary_header.total_header_length], 8, image_structure_record.columns_count, image_structure_record.lines_count, 1);
+                        saveImageP(lmeta, image);
                     }
                 }
             }
             // Check if this EMWIN data
-            else if (write_emwin && primary_header.file_type_code == 2 && (noaa_header.product_id == 9 || noaa_header.product_id == 6))
+            else if (primary_header.file_type_code == 2 && (noaa_header.product_id == 9 || noaa_header.product_id == 6))
             {
-                std::string clean_filename = current_filename.substr(0, current_filename.size() - 5); // Remove extensions
+                if (!write_emwin)
+                    return;
 
-                if (noaa_header.noaa_specific_compression == 0) // Uncompressed TXT
+                std::string clean_filename = current_filename.substr(0, current_filename.size() - 5); // Remove extensions
+                if (noaa_header.noaa_specific_compression == 0)                                       // Uncompressed TXT
                 {
                     if (!std::filesystem::exists(directory + "/EMWIN"))
                         std::filesystem::create_directory(directory + "/EMWIN");
