@@ -3,6 +3,7 @@
 #include "logger.h"
 #include <filesystem>
 #include "imgui/imgui.h"
+#include "core/config.h"
 #include "common/utils.h"
 #include "decode_utils.h"
 #include "common/dsp/io/wav_writer.h"
@@ -35,6 +36,11 @@ namespace inmarsat
                 do_save_files = parameters["save_files"].get<bool>();
             else
                 do_save_files = true;
+
+            if (parameters.contains("station_id"))
+                d_station_id = parameters["station_id"].get<std::string>();
+
+            play_audio = satdump::config::main_cfg["user_interface"]["play_audio"]["value"].get<bool>();
         }
 
         std::vector<ModuleDataType> AeroParserModule::getInputTypes()
@@ -77,7 +83,14 @@ namespace inmarsat
                 {
                     try
                     {
-                        std::string m = msg.dump();
+                        nlohmann::json msg2 = msg;
+                        if (d_station_id != ""){
+                            msg2["source"]["station_id"] = d_station_id;
+                            msg2["source"]["app"]["name"] = "SatDump";
+                            msg2["source"]["app"]["version"] = (std::string)SATDUMP_VERSION;
+                        }
+                        
+                        std::string m = msg2.dump();
                         c->send((uint8_t *)m.data(), m.size());
                     }
                     catch (std::exception &e)
@@ -292,7 +305,7 @@ namespace inmarsat
                     file_wav->write((char *)audio_out, 320 * 25);
                     final_wav_size += 320 * 25;
 
-                    if (enable_audio)
+                    if (enable_audio && play_audio)
                         audio_sink->push_samples(audio_out, 160 * 25);
                 }
                 else
@@ -312,7 +325,7 @@ namespace inmarsat
                 if (time(NULL) % 10 == 0 && lastTime != time(NULL))
                 {
                     lastTime = time(NULL);
-                    logger->info("Progress " + std::to_string(round(((float)progress / (float)filesize) * 1000.0f) / 10.0f) + "%%");
+                    logger->info("Progress " + std::to_string(round(((double)progress / (double)filesize) * 1000.0) / 10.0) + "%%");
                 }
             }
 
@@ -342,12 +355,38 @@ namespace inmarsat
 
             ImGui::Text("Decoded packets can be seen in a floating window.");
             ImGui::Spacing();
-            ImGui::TextColored(ImColor(255, 0, 0), "Note : Still WIP!");
+            ImGui::TextColored(style::theme.red, "Note : Still WIP!");
 
             ImGui::Text("Do remember you should not keep nor share data that is\nnot intended for you.");
 
+            if (enable_audio)
+            {
+                ImGui::Spacing();
+                const char *btn_icon, *label;
+                ImVec4 color;
+                if (play_audio)
+                {
+                    color = style::theme.green.Value;
+                    btn_icon = u8"\uF028##aeroaudio";
+                    label = "Audio Playing";
+                }
+                else
+                {
+                    color = style::theme.red.Value;
+                    btn_icon = u8"\uF026##aeroaudio";
+                    label = "Audio Muted";
+                }
+
+                ImGui::PushStyleColor(ImGuiCol_Text, color);
+                if (ImGui::Button(btn_icon))
+                    play_audio = !play_audio;
+                ImGui::PopStyleColor();
+                ImGui::SameLine();
+                ImGui::TextUnformatted(label);
+            }
+
             if (input_data_type == DATA_FILE)
-                ImGui::ProgressBar((float)progress / (float)filesize, ImVec2(ImGui::GetWindowWidth() - 10, 20 * ui_scale));
+                ImGui::ProgressBar((double)progress / (double)filesize, ImVec2(ImGui::GetContentRegionAvail().x, 20 * ui_scale));
 
             ImGui::End();
 
@@ -359,6 +398,7 @@ namespace inmarsat
                 ImGui::BeginTabBar("##aeromessagestabbar");
                 if (ImGui::BeginTabItem("ACARS"))
                 {
+                    float wrap_pos = ImGui::GetContentRegionMax().x;
                     ImGui::BeginTable("##aeroacardstable", 3, ImGuiTableFlags_Borders | ImGuiTableFlags_RowBg | ImGuiTableFlags_SizingFixedFit);
                     ImGui::TableSetupColumn("Plane", ImGuiTableColumnFlags_NoResize, 150 * ui_scale);
                     ImGui::TableSetupColumn("Timestamp", ImGuiTableColumnFlags_NoResize, 75 * ui_scale);
@@ -372,13 +412,15 @@ namespace inmarsat
                         {
                             ImGui::TableNextRow();
                             ImGui::TableSetColumnIndex(0);
-                            ImGui::TextColored(ImColor(160, 160, 255), "%s", msg["plane_reg"].get<std::string>().c_str());
+                            ImGui::TextColored(style::theme.lavender, "%s", msg["plane_reg"].get<std::string>().c_str());
                             ImGui::TableSetColumnIndex(1);
-                            ImGui::TextColored(ImColor(255, 255, 0), "%s", timestampToTod(msg["timestamp"].get<double>()).c_str());
+                            ImGui::TextColored(style::theme.yellow, "%s", timestampToTod(msg["timestamp"].get<double>()).c_str());
                             ImGui::TableSetColumnIndex(2);
-                            ImGui::TextColored(ImColor(0, 255, 0), "%s", msg["message"].get<std::string>().c_str());
+                            ImGui::PushTextWrapPos(wrap_pos);
+                            ImGui::TextColored(style::theme.green, "%s", msg["message"].get<std::string>().c_str());
+                            ImGui::PopTextWrapPos();
                         }
-                        catch (std::exception&)
+                        catch (std::exception &)
                         {
                         }
                     }
@@ -387,6 +429,7 @@ namespace inmarsat
                 }
                 if (ImGui::BeginTabItem("Packets"))
                 {
+                    float wrap_pos = ImGui::GetContentRegionMax().x;
                     ImGui::BeginTable("##aeromessagetable", 3, ImGuiTableFlags_Borders | ImGuiTableFlags_RowBg | ImGuiTableFlags_SizingFixedFit);
                     ImGui::TableSetupColumn("Type", ImGuiTableColumnFlags_NoResize, 150 * ui_scale);
                     ImGui::TableSetupColumn("Timestamp", ImGuiTableColumnFlags_NoResize, 75 * ui_scale);
@@ -400,13 +443,15 @@ namespace inmarsat
                         {
                             ImGui::TableNextRow();
                             ImGui::TableSetColumnIndex(0);
-                            ImGui::TextColored(ImColor(160, 160, 255), "%s", msg["msg_name"].get<std::string>().c_str());
+                            ImGui::TextColored(style::theme.lavender, "%s", msg["msg_name"].get<std::string>().c_str());
                             ImGui::TableSetColumnIndex(1);
-                            ImGui::TextColored(ImColor(255, 255, 0), "%s", timestampToTod(msg["timestamp"].get<double>()).c_str());
+                            ImGui::TextColored(style::theme.yellow, "%s", timestampToTod(msg["timestamp"].get<double>()).c_str());
                             ImGui::TableSetColumnIndex(2);
-                            ImGui::TextColored(ImColor(0, 255, 0), "%s", msg.dump().c_str());
+                            ImGui::PushTextWrapPos(wrap_pos);
+                            ImGui::TextColored(style::theme.green, "%s", msg.dump().c_str());
+                            ImGui::PopTextWrapPos();
                         }
-                        catch (std::exception&)
+                        catch (std::exception &)
                         {
                         }
                     }
