@@ -26,7 +26,7 @@ namespace jpss
             int lines;
             std::vector<double> timestamps;
             void work(ccsds::CCSDSPacket &packet);
-            image::Image<uint16_t> getChannel(int channel);
+            image::Image getChannel(int channel);
         };
     } // namespace atms
 } // namespace jpss
\ No newline at end of file
