@@ -4,10 +4,12 @@
 
 #include "meteor/module_meteor_hrpt_decoder.h"
 #include "meteor/module_meteor_lrpt_decoder.h"
-#include "meteor/module_meteor_dump_decoder.h"
+#include "meteor/module_meteor_xband_decoder.h"
 #include "meteor/instruments/msumr/module_meteor_msumr_lrpt.h"
 #include "meteor/module_meteor_instruments.h"
-#include "meteor/module_meteor_dump_instruments.h"
+#include "meteor/module_meteor_xband_instruments.h"
+
+#include "meteor/module_meteor_qpsk_kmss_decoder.h"
 
 class MeteorSupport : public satdump::Plugin
 {
@@ -26,10 +28,12 @@ public:
     {
         REGISTER_MODULE_EXTERNAL(evt.modules_registry, meteor::METEORHRPTDecoderModule);
         REGISTER_MODULE_EXTERNAL(evt.modules_registry, meteor::METEORLRPTDecoderModule);
-        REGISTER_MODULE_EXTERNAL(evt.modules_registry, meteor::METEORDumpDecoderModule);
+        REGISTER_MODULE_EXTERNAL(evt.modules_registry, meteor::MeteorXBandDecoderModule);
         REGISTER_MODULE_EXTERNAL(evt.modules_registry, meteor::msumr::METEORMSUMRLRPTDecoderModule);
         REGISTER_MODULE_EXTERNAL(evt.modules_registry, meteor::instruments::MeteorInstrumentsDecoderModule);
-        REGISTER_MODULE_EXTERNAL(evt.modules_registry, meteor::instruments::MeteorDumpInstrumentsDecoderModule);
+        REGISTER_MODULE_EXTERNAL(evt.modules_registry, meteor::instruments::MeteorXBandInstrumentsDecoderModule);
+
+        REGISTER_MODULE_EXTERNAL(evt.modules_registry, meteor::MeteorQPSKKmssDecoderModule);
     }
 };
 
