@@ -1,12 +1,9 @@
 #pragma once
-
-#include "common/ccsds/ccsds.h"
-#include <tuple>
-
-#include "common/image/image.h"
-
-#include "lrpt/segment.h"
+#include <set>
 #include <array>
+#include "common/ccsds/ccsds.h"
+#include "common/image/image.h"
+#include "lrpt/segment.h"
 
 namespace meteor
 {
@@ -17,9 +14,8 @@ namespace meteor
             class MSUMRReader
             {
             private:
-                unsigned char *channels[6];
                 Segment *segments[6];
-                uint32_t rollover[6], lastSeq[6], offset[6], firstSeg[6], lastSeg[6], segCount[6];
+                uint32_t firstSeg[6], rollover[6], lastSeq[6], offset[6], lastSeg[6];
 
                 time_t dayValue;
 
@@ -29,7 +25,7 @@ namespace meteor
                 MSUMRReader(bool meteorm2x_mode);
                 ~MSUMRReader();
                 void work(ccsds::CCSDSPacket &packet);
-                image::Image<uint8_t> getChannel(int channel, int32_t first = -1, int32_t last = -1, int32_t offset = 1);
+                image::Image getChannel(int channel, size_t max_correct = 0, int32_t first = -1, int32_t last = -1, int32_t offset = 1);
                 std::array<int32_t, 3> correlateChannels(int channel1, int channel2);
                 std::array<int32_t, 3> correlateChannels(int channel1, int channel2, int channel3);
 
