@@ -1,6 +1,7 @@
 #pragma once
 
 #include <cstdint>
+#include <vector>
 #include "common/image/image.h"
 
 namespace meteor
@@ -17,7 +18,7 @@ namespace meteor
             ~MSUMRReader();
             int lines;
             void work(uint8_t *buffer);
-            image::Image<uint16_t> getChannel(int channel);
+            image::Image getChannel(int channel);
         };
     } // namespace msumr
 } // namespace meteor
\ No newline at end of file
