@@ -3,7 +3,7 @@
 #include "common/codings/rotation.h"
 #include "common/codings/randomization.h"
 #include "common/codings/differential/nrzm.h"
-#include "imgui/imgui.h"
+#include "common/widgets/themed_widgets.h"
 #include "common/codings/viterbi/viterbi27.h"
 #include "common/codings/correlator.h"
 #include "common/codings/reedsolomon/reedsolomon.h"
@@ -14,7 +14,7 @@
 #define ENCODED_FRAME_SIZE 1024 * 8 * 2
 
 // Return filesize
-size_t getFilesize(std::string filepath);
+uint64_t getFilesize(std::string filepath);
 
 namespace meteor
 {
@@ -238,7 +238,7 @@ namespace meteor
                     lastTime = time(NULL);
                     std::string viterbi_state = viterbi_lock == 0 ? "NOSYNC" : "SYNCED";
                     std::string deframer_state = deframer->getState() == deframer->STATE_NOSYNC ? "NOSYNC" : (deframer->getState() == deframer->STATE_SYNCING ? "SYNCING" : "SYNCED");
-                    logger->info("Progress " + std::to_string(round(((float)progress / (float)filesize) * 1000.0f) / 10.0f) + "%%, Viterbi : " + viterbi_state + " BER : " + std::to_string(viterbi_ber) + ", Deframer : " + deframer_state);
+                    logger->info("Progress " + std::to_string(round(((double)progress / (double)filesize) * 1000.0) / 10.0) + "%%, Viterbi : " + viterbi_state + " BER : " + std::to_string(viterbi_ber) + ", Deframer : " + deframer_state);
                 }
             }
 
@@ -329,7 +329,7 @@ namespace meteor
                 {
                     lastTime = time(NULL);
                     std::string lock_state = locked ? "SYNCED" : "NOSYNC";
-                    logger->info("Progress " + std::to_string(round(((float)progress / (float)filesize) * 1000.0f) / 10.0f) + "%%, Viterbi BER : " + std::to_string(viterbin->ber() * 100) + "%%, Lock : " + lock_state);
+                    logger->info("Progress " + std::to_string(round(((double)progress / (double)filesize) * 1000.0) / 10.0) + "%%, Viterbi BER : " + std::to_string(viterbi->ber() * 100) + "%%, Lock : " + lock_state);
                 }
             }
         }
@@ -350,14 +350,14 @@ namespace meteor
                 ImDrawList *draw_list = ImGui::GetWindowDrawList();
                 draw_list->AddRectFilled(ImGui::GetCursorScreenPos(),
                                          ImVec2(ImGui::GetCursorScreenPos().x + 200 * ui_scale, ImGui::GetCursorScreenPos().y + 200 * ui_scale),
-                                         ImColor::HSV(0, 0, 0));
+                                         style::theme.widget_bg);
 
                 for (int i = 0; i < 2048; i++)
                 {
                     draw_list->AddCircleFilled(ImVec2(ImGui::GetCursorScreenPos().x + (int)(100 * ui_scale + (((int8_t *)buffer)[i * 2 + 0] / 127.0) * 100 * ui_scale) % int(200 * ui_scale),
                                                       ImGui::GetCursorScreenPos().y + (int)(100 * ui_scale + (((int8_t *)buffer)[i * 2 + 1] / 127.0) * 100 * ui_scale) % int(200 * ui_scale)),
                                                2 * ui_scale,
-                                               ImColor::HSV(113.0 / 360.0, 1, 1, 1.0));
+                                               style::theme.constellation);
                 }
 
                 ImGui::Dummy(ImVec2(200 * ui_scale + 3, 200 * ui_scale + 3));
@@ -380,18 +380,19 @@ namespace meteor
                     ImGui::SameLine();
 
                     if (viterbi_lock == 0)
-                        ImGui::TextColored(IMCOLOR_NOSYNC, "NOSYNC");
+                        ImGui::TextColored(style::theme.red, "NOSYNC");
                     else
-                        ImGui::TextColored(IMCOLOR_SYNCED, "SYNCED");
+                        ImGui::TextColored(style::theme.green, "SYNCED");
 
                     ImGui::Text("BER   : ");
                     ImGui::SameLine();
-                    ImGui::TextColored(viterbi_lock == 0 ? IMCOLOR_NOSYNC : IMCOLOR_SYNCED, UITO_C_STR(ber));
+                    ImGui::TextColored(viterbi_lock == 0 ? style::theme.red : style::theme.green, UITO_C_STR(ber));
 
                     std::memmove(&ber_history[0], &ber_history[1], (200 - 1) * sizeof(float));
                     ber_history[200 - 1] = ber;
 
-                    ImGui::PlotLines("", ber_history, IM_ARRAYSIZE(ber_history), 0, "", 0.0f, 1.0f, ImVec2(200 * ui_scale, 50 * ui_scale));
+                    widgets::ThemedPlotLines(style::theme.plot_bg.Value, "", ber_history, IM_ARRAYSIZE(ber_history), 0, "", 0.0f, 1.0f,
+                        ImVec2(200 * ui_scale, 50 * ui_scale));
                 }
 
                 ImGui::Spacing();
@@ -403,11 +404,11 @@ namespace meteor
                     ImGui::SameLine();
 
                     if (deframer->getState() == deframer->STATE_NOSYNC)
-                        ImGui::TextColored(IMCOLOR_NOSYNC, "NOSYNC");
+                        ImGui::TextColored(style::theme.red, "NOSYNC");
                     else if (deframer->getState() == deframer->STATE_SYNCING)
-                        ImGui::TextColored(IMCOLOR_SYNCING, "SYNCING");
+                        ImGui::TextColored(style::theme.orange, "SYNCING");
                     else
-                        ImGui::TextColored(IMCOLOR_SYNCED, "SYNCED");
+                        ImGui::TextColored(style::theme.green, "SYNCED");
                 }
 
                 ImGui::Spacing();
@@ -420,11 +421,11 @@ namespace meteor
                         ImGui::SameLine();
 
                         if (errors[i] == -1)
-                            ImGui::TextColored(IMCOLOR_NOSYNC, "%i ", i);
+                            ImGui::TextColored(style::theme.red, "%i ", i);
                         else if (errors[i] > 0)
-                            ImGui::TextColored(IMCOLOR_SYNCING, "%i ", i);
+                            ImGui::TextColored(style::theme.orange, "%i ", i);
                         else
-                            ImGui::TextColored(IMCOLOR_SYNCED, "%i ", i);
+                            ImGui::TextColored(style::theme.green, "%i ", i);
                     }
                 }
             }
@@ -436,12 +437,13 @@ namespace meteor
                 {
                     ImGui::Text("Corr  : ");
                     ImGui::SameLine();
-                    ImGui::TextColored(locked ? IMCOLOR_SYNCED : IMCOLOR_SYNCING, UITO_C_STR(cor));
+                    ImGui::TextColored(locked ? style::theme.green : style::theme.orange, UITO_C_STR(cor));
 
                     std::memmove(&cor_history[0], &cor_history[1], (200 - 1) * sizeof(float));
                     cor_history[200 - 1] = cor;
 
-                    ImGui::PlotLines("", cor_history, IM_ARRAYSIZE(cor_history), 0, "", 40.0f, 64.0f, ImVec2(200 * ui_scale, 50 * ui_scale));
+                    widgets::ThemedPlotLines(style::theme.plot_bg.Value, "", cor_history, IM_ARRAYSIZE(cor_history), 0, "", 40.0f, 64.0f,
+                        ImVec2(200 * ui_scale, 50 * ui_scale));
                 }
 
                 ImGui::Spacing();
@@ -450,12 +452,13 @@ namespace meteor
                 {
                     ImGui::Text("BER   : ");
                     ImGui::SameLine();
-                    ImGui::TextColored(ber < 0.22 ? IMCOLOR_SYNCED : IMCOLOR_NOSYNC, UITO_C_STR(ber));
+                    ImGui::TextColored(ber < 0.22 ? style::theme.green : style::theme.red, UITO_C_STR(ber));
 
                     std::memmove(&ber_history[0], &ber_history[1], (200 - 1) * sizeof(float));
                     ber_history[200 - 1] = ber;
 
-                    ImGui::PlotLines("", ber_history, IM_ARRAYSIZE(ber_history), 0, "", 0.0f, 1.0f, ImVec2(200 * ui_scale, 50 * ui_scale));
+                    widgets::ThemedPlotLines(style::theme.plot_bg.Value, "", ber_history, IM_ARRAYSIZE(ber_history), 0, "", 0.0f, 1.0f,
+                        ImVec2(200 * ui_scale, 50 * ui_scale));
                 }
 
                 ImGui::Spacing();
@@ -468,11 +471,11 @@ namespace meteor
                         ImGui::SameLine();
 
                         if (errors[i] == -1)
-                            ImGui::TextColored(IMCOLOR_NOSYNC, "%i ", i);
+                            ImGui::TextColored(style::theme.red, "%i ", i);
                         else if (errors[i] > 0)
-                            ImGui::TextColored(IMCOLOR_SYNCING, "%i ", i);
+                            ImGui::TextColored(style::theme.orange, "%i ", i);
                         else
-                            ImGui::TextColored(IMCOLOR_SYNCED, "%i ", i);
+                            ImGui::TextColored(style::theme.green, "%i ", i);
                     }
                 }
             }
@@ -480,7 +483,7 @@ namespace meteor
         ImGui::EndGroup();
 
         if (!streamingInput)
-            ImGui::ProgressBar((float)progress / (float)filesize, ImVec2(ImGui::GetWindowWidth() - 10, 20 * ui_scale));
+            ImGui::ProgressBar((double)progress / (double)filesize, ImVec2(ImGui::GetContentRegionAvail().x, 20 * ui_scale));
 
         ImGui::End();
     }
