@@ -2,6 +2,7 @@
 
 #include "products/image_products.h"
 #include "nlohmann/json.hpp"
+#include "core/exception.h"
 
 class NoaaMHSCalibrator : public satdump::ImageProducts::CalibratorBase
 {
@@ -23,10 +24,12 @@ public:
 
     void init()
     {
+        if (!d_calib.contains("vars") || !d_calib["vars"].contains("perLine_perChannel"))
+            throw satdump_exception("Calibration data missing!");
         perLine_perChannel = d_calib["vars"]["perLine_perChannel"];
     }
 
-    double compute(int channel, int pos_x, int pos_y, int px_val)
+    double compute(int channel, int /*pos_x*/, int pos_y, int px_val)
     {
         try
         {
