@@ -18,7 +18,7 @@ namespace metop
             int lines;
             std::vector<double> timestamps;
             void work(ccsds::CCSDSPacket &packet);
-            image::Image<uint16_t> getChannel(int channel);
+            image::Image getChannel(int channel);
         };
     } // namespace iasi
 } // namespace metop
\ No newline at end of file
