@@ -14,6 +14,9 @@
 #include "products/dataset.h"
 #include "common/tracking/tle.h"
 #include "resources.h"
+#include "nlohmann/json_utils.h"
+#include "common/image/io.h"
+#include "common/image/processing.h"
 
 namespace metop
 {
@@ -23,6 +26,7 @@ namespace metop
             : ProcessingModule(input_file, output_file_hint, parameters), avhrr_reader(false, -1)
         {
             write_hpt = parameters.contains("write_hpt") ? parameters["write_hpt"].get<bool>() : false;
+            ignore_integrated_tle = parameters.contains("ignore_integrated_tle") ? parameters["ignore_integrated_tle"].get<bool>() : false;
         }
 
         void MetOpInstrumentsDecoderModule::process()
@@ -138,13 +142,13 @@ namespace metop
                 if (time(NULL) % 10 == 0 && lastTime != time(NULL))
                 {
                     lastTime = time(NULL);
-                    logger->info("Progress " + std::to_string(round(((float)progress / (float)filesize) * 1000.0f) / 10.0f) + "%%");
+                    logger->info("Progress " + std::to_string(round(((double)progress / (double)filesize) * 1000.0) / 10.0) + "%%");
                 }
             }
 
             data_in.close();
 
-            int scid = most_common(metop_scids.begin(), metop_scids.end());
+            int scid = most_common(metop_scids.begin(), metop_scids.end(), 0);
             metop_scids.clear();
 
             std::string sat_name = "Unknown MetOp";
@@ -164,7 +168,7 @@ namespace metop
                 norad = METOP_C_NORAD;
 
             std::optional<satdump::TLE> satellite_tle = admin_msg_reader.tles.get_from_norad(norad);
-            if (!satellite_tle.has_value())
+            if (!satellite_tle.has_value() || ignore_integrated_tle)
                 satellite_tle = satdump::general_tle_registry.get_from_norad(norad);
 
             // Products dataset
@@ -303,7 +307,7 @@ namespace metop
 
                 // Output a few nice composites as well
                 logger->info("ASCAT Composite...");
-                image::Image<uint16_t> imageAll(256 * 2, ascat_reader.getChannelImg(0).height() * 3, 1);
+                image::Image imageAll(16, 256 * 2, ascat_reader.getChannelImg(0).height() * 3, 1);
                 {
                     int height = ascat_reader.getChannelImg(0).height();
 
@@ -330,7 +334,7 @@ namespace metop
                     imageAll.draw_image(0, image1, 256 * 1, height * 2);
                 }
 
-                WRITE_IMAGE(imageAll, directory + "/ASCAT-ALL");
+                image::save_img(imageAll, directory + "/ASCAT-ALL");
 
                 ascat_products.save(directory);
                 dataset.products_list.push_back("ASCAT");
@@ -360,9 +364,9 @@ namespace metop
                 if (iasi_reader_img.lines > 0)
                 {
                     logger->info("Channel IR imaging...");
-                    image::Image<uint16_t> iasi_imaging = iasi_reader_img.getIRChannel();
+                    image::Image iasi_imaging = iasi_reader_img.getIRChannel();
                     iasi_imaging = image::bowtie::correctGenericBowTie(iasi_imaging, 1, scanHeight, alpha, beta); // Bowtie.... As IASI scans per IFOV
-                    iasi_imaging.simple_despeckle(10);                                                            // And, it has some dead pixels sometimes so well, we need to remove them I guess?
+                    image::simple_despeckle(iasi_imaging, 10);                                                    // And, it has some dead pixels sometimes so well, we need to remove them I guess?
 
                     // Test! TODO : Cleanup!!
                     satdump::ImageProducts iasi_img_products;
@@ -421,7 +425,7 @@ namespace metop
                 amsu_products.set_proj_cfg(loadJsonFile(resources::getResourcePath("projections_settings/metop_abc_amsu.json")));
 
                 for (int i = 0; i < 15; i++)
-                    amsu_products.images.push_back({"AMSU-A-" + std::to_string(i + 1) + ".png", std::to_string(i + 1), amsu_reader.getChannel(i), i < 2 ? amsu_reader.timestamps_A2 : amsu_reader.timestamps_A1});
+                    amsu_products.images.push_back({"AMSU-A-" + std::to_string(i + 1), std::to_string(i + 1), amsu_reader.getChannel(i), i < 2 ? amsu_reader.timestamps_A2 : amsu_reader.timestamps_A1});
 
                 // calib
                 nlohmann::json calib_coefs = loadJsonFile(resources::getResourcePath("calibration/AMSU-A.json"));
@@ -526,7 +530,7 @@ namespace metop
                 ImGui::TableSetColumnIndex(0);
                 ImGui::Text("AVHRR");
                 ImGui::TableSetColumnIndex(1);
-                ImGui::TextColored(ImColor(0, 255, 0), "%d", avhrr_reader.lines);
+                ImGui::TextColored(style::theme.green, "%d", avhrr_reader.lines);
                 ImGui::TableSetColumnIndex(2);
                 drawStatus(avhrr_status);
 
@@ -534,7 +538,7 @@ namespace metop
                 ImGui::TableSetColumnIndex(0);
                 ImGui::Text("IASI");
                 ImGui::TableSetColumnIndex(1);
-                ImGui::TextColored(ImColor(0, 255, 0), "%d", iasi_reader.lines);
+                ImGui::TextColored(style::theme.green, "%d", iasi_reader.lines);
                 ImGui::TableSetColumnIndex(2);
                 drawStatus(iasi_status);
 
@@ -542,7 +546,7 @@ namespace metop
                 ImGui::TableSetColumnIndex(0);
                 ImGui::Text("IASI Imaging");
                 ImGui::TableSetColumnIndex(1);
-                ImGui::TextColored(ImColor(0, 255, 0), "%d", iasi_reader_img.lines * 64);
+                ImGui::TextColored(style::theme.green, "%d", iasi_reader_img.lines * 64);
                 ImGui::TableSetColumnIndex(2);
                 drawStatus(iasi_img_status);
 
@@ -550,7 +554,7 @@ namespace metop
                 ImGui::TableSetColumnIndex(0);
                 ImGui::Text("MHS");
                 ImGui::TableSetColumnIndex(1);
-                ImGui::TextColored(ImColor(0, 255, 0), "%d", mhs_reader.line);
+                ImGui::TextColored(style::theme.green, "%d", mhs_reader.line);
                 ImGui::TableSetColumnIndex(2);
                 drawStatus(mhs_status);
 
@@ -558,7 +562,7 @@ namespace metop
                 ImGui::TableSetColumnIndex(0);
                 ImGui::Text("AMSU A1");
                 ImGui::TableSetColumnIndex(1);
-                ImGui::TextColored(ImColor(0, 255, 0), "%d", amsu_reader.linesA1);
+                ImGui::TextColored(style::theme.green, "%d", amsu_reader.linesA1);
                 ImGui::TableSetColumnIndex(2);
                 drawStatus(amsu_status);
 
@@ -566,7 +570,7 @@ namespace metop
                 ImGui::TableSetColumnIndex(0);
                 ImGui::Text("AMSU A2");
                 ImGui::TableSetColumnIndex(1);
-                ImGui::TextColored(ImColor(0, 255, 0), "%d", amsu_reader.linesA2);
+                ImGui::TextColored(style::theme.green, "%d", amsu_reader.linesA2);
                 ImGui::TableSetColumnIndex(2);
                 drawStatus(amsu_status);
 
@@ -574,7 +578,7 @@ namespace metop
                 ImGui::TableSetColumnIndex(0);
                 ImGui::Text("GOME");
                 ImGui::TableSetColumnIndex(1);
-                ImGui::TextColored(ImColor(0, 255, 0), "%d", gome_reader.lines);
+                ImGui::TextColored(style::theme.green, "%d", gome_reader.lines);
                 ImGui::TableSetColumnIndex(2);
                 drawStatus(gome_status);
 
@@ -582,7 +586,7 @@ namespace metop
                 ImGui::TableSetColumnIndex(0);
                 ImGui::Text("ASCAT");
                 ImGui::TableSetColumnIndex(1);
-                ImGui::TextColored(ImColor(0, 255, 0), "%d", ascat_reader.lines[0]);
+                ImGui::TextColored(style::theme.green, "%d", ascat_reader.lines[0]);
                 ImGui::TableSetColumnIndex(2);
                 drawStatus(gome_status);
 
@@ -590,7 +594,7 @@ namespace metop
                 ImGui::TableSetColumnIndex(0);
                 ImGui::Text("SEM");
                 ImGui::TableSetColumnIndex(1);
-                ImGui::TextColored(ImColor(0, 255, 0), "%d", sem_reader.samples);
+                ImGui::TextColored(style::theme.green, "%d", sem_reader.samples);
                 ImGui::TableSetColumnIndex(2);
                 drawStatus(sem_status);
 
@@ -598,14 +602,14 @@ namespace metop
                 ImGui::TableSetColumnIndex(0);
                 ImGui::Text("Admin Messages");
                 ImGui::TableSetColumnIndex(1);
-                ImGui::TextColored(ImColor(0, 255, 0), "%d", admin_msg_reader.count);
+                ImGui::TextColored(style::theme.green, "%d", admin_msg_reader.count);
                 ImGui::TableSetColumnIndex(2);
                 drawStatus(admin_msg_status);
 
                 ImGui::EndTable();
             }
 
-            ImGui::ProgressBar((float)progress / (float)filesize, ImVec2(ImGui::GetWindowWidth() - 10, 20 * ui_scale));
+            ImGui::ProgressBar((double)progress / (double)filesize, ImVec2(ImGui::GetContentRegionAvail().x, 20 * ui_scale));
 
             ImGui::End();
         }
