@@ -18,9 +18,10 @@ namespace noaa
             : ProcessingModule(input_file, output_file_hint, parameters),
               is_gac(parameters.count("gac_mode") > 0 ? parameters["gac_mode"].get<bool>() : 0),
               is_dsb(parameters.count("dsb_mode") > 0 ? parameters["dsb_mode"].get<bool>() : 0),
-              avhrr_reader(is_gac, parameters["year_override"].get<int>()),
-              hirs_reader(parameters["year_override"].get<int>()),
-              sem_reader(parameters["year_override"].get<int>())
+              avhrr_reader(is_gac, parameters.count("year_override") > 0 ? parameters["year_override"].get<int>() : -1),
+              hirs_reader(parameters.count("year_override") > 0 ? parameters["year_override"].get<int>() : -1),
+              sem_reader(parameters.count("year_override") > 0 ? parameters["year_override"].get<int>() : -1),
+              telemetry_reader(parameters.count("year_override") > 0 ? parameters["year_override"].get<int>() : -1)
         {
         }
 
@@ -61,6 +62,7 @@ namespace noaa
                                     {
                                         hirs_reader.work(frameBuffer);
                                         sem_reader.work(frameBuffer);
+                                        telemetry_reader.work(frameBuffer);
 
                                         // push the timestamps to AIP, since it depends on TIP for that (ridiculous)
                                         amsu_reader.last_TIP_timestamp = hirs_reader.last_timestamp;
@@ -92,6 +94,7 @@ namespace noaa
                                 }
                                 hirs_reader.work(frameBuffer);
                                 sem_reader.work(frameBuffer);
+                                telemetry_reader.work(frameBuffer);
                                 // push the timestamps to AIP, since it depends on TIP for that (ridiculous)
                                 amsu_reader.last_TIP_timestamp = hirs_reader.last_timestamp;
 
@@ -110,12 +113,12 @@ namespace noaa
                     if (time(NULL) % 10 == 0 && lastTime != time(NULL))
                     {
                         lastTime = time(NULL);
-                        logger->info("Progress " + std::to_string(round(((float)progress / (float)filesize) * 1000.0f) / 10.0f) + "%%");
+                        logger->info("Progress " + std::to_string(round(((double)progress / (double)filesize) * 1000.0) / 10.0) + "%%");
                     }
                 } // done with the frames
                 data_in.close();
 
-                int scid = most_common(spacecraft_ids.begin(), spacecraft_ids.end());
+                int scid = most_common(spacecraft_ids.begin(), spacecraft_ids.end(), 0);
                 spacecraft_ids.clear();
                 int norad = 0;
                 std::string sat_name = "Unknown NOAA";
@@ -219,6 +222,7 @@ namespace noaa
                     if (scid == 15) // NOAA-19
                         proj_settings = loadJsonFile(resources::getResourcePath("projections_settings/noaa_19_avhrr.json"));
 
+                    proj_settings.erase("apt_marker_offset");
                     if (is_gac)
                         proj_settings["image_width"] = 409;
 
@@ -274,7 +278,7 @@ namespace noaa
                         std::filesystem::create_directory(directory);
 
                     logger->info("----------- SEM");
-                    logger->info("Sample counts from selected cahnnels :");
+                    logger->info("Sample counts from selected channels :");
                     logger->info("Channel OP1   : " + std::to_string(sem_reader.getChannel(0).size()));
                     logger->info("Channel P8    : " + std::to_string(sem_reader.getChannel(20).size()));
                     logger->info("Channel 0DE1  : " + std::to_string(sem_reader.getChannel(22).size()));
@@ -296,6 +300,17 @@ namespace noaa
                     sem_status = DONE;
                 }
 
+                // telemetry
+                {
+                    telemetry_status = SAVING;
+                    std::string directory = d_output_file_hint.substr(0, d_output_file_hint.rfind('/')) + "/telemetry";
+                    if (!std::filesystem::exists(directory))
+                        std::filesystem::create_directory(directory);
+
+                    saveJsonFile(directory + "/telem.json", telemetry_reader.dump_telemetry());
+                    telemetry_status = DONE;
+                }
+
                 // MHS
                 if (scid == 15)
                 { // only N19 has operational MHS
@@ -401,18 +416,19 @@ namespace noaa
                     data_in.read((char *)&buffer[0], 104);
                     hirs_reader.work(buffer);
                     sem_reader.work(buffer);
+                    telemetry_reader.work(buffer);
                     scid_list.push_back(buffer[2] & 0b00001111);
 
                     progress = data_in.tellg();
                     if (time(NULL) % 10 == 0 && lastTime != time(NULL))
                     {
                         lastTime = time(NULL);
-                        logger->info("Progress " + std::to_string(round(((float)progress / (float)filesize) * 1000.0f) / 10.0f) + "%%");
+                        logger->info("Progress " + std::to_string(round(((double)progress / (double)filesize) * 1000.0) / 10.0) + "%%");
                     }
                 }
 
                 // ID
-                uint8_t scid = most_common(scid_list.begin(), scid_list.end());
+                uint8_t scid = most_common(scid_list.begin(), scid_list.end(), 0);
                 scid_list.clear();
                 int norad = 0;
                 std::string sat_name = "Unknown NOAA";
@@ -483,7 +499,7 @@ namespace noaa
                         std::filesystem::create_directory(directory);
 
                     logger->info("----------- SEM");
-                    logger->info("Sample counts from selected cahnnels :");
+                    logger->info("Sample counts from selected channels :");
                     logger->info("Channel OP1   : " + std::to_string(sem_reader.getChannel(0).size()));
                     logger->info("Channel P8    : " + std::to_string(sem_reader.getChannel(20).size()));
                     logger->info("Channel 0DE1  : " + std::to_string(sem_reader.getChannel(22).size()));
@@ -504,6 +520,17 @@ namespace noaa
                     dataset.products_list.push_back("SEM");
                     sem_status = DONE;
                 }
+
+                // telemetry
+                {
+                    telemetry_status = SAVING;
+                    std::string directory = d_output_file_hint.substr(0, d_output_file_hint.rfind('/')) + "/telemetry";
+                    if (!std::filesystem::exists(directory))
+                        std::filesystem::create_directory(directory);
+
+                    saveJsonFile(directory + "/telem.json", telemetry_reader.dump_telemetry());
+                    telemetry_status = DONE;
+                }
                 dataset.save(d_output_file_hint.substr(0, d_output_file_hint.rfind('/')));
             }
         }
@@ -526,7 +553,7 @@ namespace noaa
                 ImGui::TableSetColumnIndex(0);
                 ImGui::Text("AVHRR");
                 ImGui::TableSetColumnIndex(1);
-                ImGui::TextColored(ImColor(0, 255, 0), "%d", avhrr_reader.lines);
+                ImGui::TextColored(style::theme.green, "%d", avhrr_reader.lines);
                 ImGui::TableSetColumnIndex(2);
                 drawStatus(avhrr_status);
 
@@ -534,7 +561,7 @@ namespace noaa
                 ImGui::TableSetColumnIndex(0);
                 ImGui::Text("HIRS");
                 ImGui::TableSetColumnIndex(1);
-                ImGui::TextColored(ImColor(0, 255, 0), "%d", hirs_reader.line);
+                ImGui::TextColored(style::theme.green, "%d", hirs_reader.line);
                 ImGui::TableSetColumnIndex(2);
                 drawStatus(hirs_status);
 
@@ -542,7 +569,7 @@ namespace noaa
                 ImGui::TableSetColumnIndex(0);
                 ImGui::Text("MHS");
                 ImGui::TableSetColumnIndex(1);
-                ImGui::TextColored(ImColor(0, 255, 0), "%d", mhs_reader.line);
+                ImGui::TextColored(style::theme.green, "%d", mhs_reader.line);
                 ImGui::TableSetColumnIndex(2);
                 drawStatus(mhs_status);
 
@@ -550,7 +577,7 @@ namespace noaa
                 ImGui::TableSetColumnIndex(0);
                 ImGui::Text("AMSU A1");
                 ImGui::TableSetColumnIndex(1);
-                ImGui::TextColored(ImColor(0, 255, 0), "%d", amsu_reader.linesA1);
+                ImGui::TextColored(style::theme.green, "%d", amsu_reader.linesA1);
                 ImGui::TableSetColumnIndex(2);
                 drawStatus(amsu_status);
 
@@ -558,14 +585,14 @@ namespace noaa
                 ImGui::TableSetColumnIndex(0);
                 ImGui::Text("AMSU A2");
                 ImGui::TableSetColumnIndex(1);
-                ImGui::TextColored(ImColor(0, 255, 0), "%d", amsu_reader.linesA2);
+                ImGui::TextColored(style::theme.green, "%d", amsu_reader.linesA2);
                 ImGui::TableSetColumnIndex(2);
                 drawStatus(amsu_status);
 
                 ImGui::EndTable();
             }
 
-            ImGui::ProgressBar((float)progress / (float)filesize, ImVec2(ImGui::GetWindowWidth() - 10, 20 * ui_scale));
+            ImGui::ProgressBar((double)progress / (double)filesize, ImVec2(ImGui::GetContentRegionAvail().x, 20 * ui_scale));
 
             ImGui::End();
         }
@@ -585,4 +612,4 @@ namespace noaa
             return std::make_shared<NOAAInstrumentsDecoderModule>(input_file, output_file_hint, parameters);
         }
     } // namespace amsu
-} // namespace noaa
\ No newline at end of file
+} // namespace noaa
