@@ -21,8 +21,8 @@ namespace spacex
 
         std::ifstream data_in;
         std::ofstream data_out;
-        std::atomic<size_t> filesize;
-        std::atomic<size_t> progress;
+        std::atomic<uint64_t> filesize;
+        std::atomic<uint64_t> progress;
 
         bool qpsk;
 
