@@ -8,8 +8,8 @@ namespace tubin
     class TUBINDecoderModule : public ProcessingModule
     {
     protected:
-        std::atomic<size_t> filesize;
-        std::atomic<size_t> progress;
+        std::atomic<uint64_t> filesize;
+        std::atomic<uint64_t> progress;
 
         bool d_check_crc;
 
