@@ -3,9 +3,12 @@
 #include "live.h"
 #include "offline.h"
 #include "record.h"
+#include "autotrack/autotrack.h"
 
 #include "sdr_probe.h"
 
+#include "project/project.h"
+
 int main(int argc, char *argv[])
 {
     // Init logger
@@ -29,6 +32,25 @@ int main(int argc, char *argv[])
         if (ret != 0)
             return ret;
     }
+    else if (std::string(argv[1]) == "autotrack")
+    {
+        int ret = main_autotrack(argc, argv);
+        if (ret != 0)
+            return ret;
+    }
+    else if (std::string(argv[1]) == "project")
+    {
+        try
+        {
+            int ret = main_project(argc - 2, argv + 2);
+            if (ret != 0)
+                return ret;
+        }
+        catch (std::exception &e)
+        {
+            logger->error("Error running project! %s", e.what());
+        }
+    }
     //////////////
     else if (std::string(argv[1]) == "sdr_probe")
     {
