@@ -1,17 +1,18 @@
 #include "baseband_interface.h"
+#include "core/exception.h"
 
 namespace dsp
 {
     BasebandType basebandTypeFromString(std::string type)
     {
-        if (type == "s16")
-            return IS_16;
-        else if (type == "s8")
-            return IS_8;
-        else if (type == "f32")
+        if (type == "cs16")
+            return CS_16;
+        else if (type == "cs8")
+            return CS_8;
+        else if (type == "cf32")
             return CF_32;
-        else if (type == "u8")
-            return IU_8;
+        else if (type == "cu8")
+            return CU_8;
         else if (type == "w16")
             return WAV_16;
 #ifdef BUILD_ZIQ
@@ -21,6 +22,6 @@ namespace dsp
         else if (type == "ziq2")
             return ZIQ2;
         else
-            throw std::runtime_error("Unknown baseband type " + type);
+            throw satdump_exception("Unknown baseband type " + type);
     }
 }
\ No newline at end of file
