@@ -3,6 +3,8 @@
 
 #include "file_source.h"
 
+#include "logger.h"
+
 namespace dsp
 {
     std::map<std::string, RegisteredSource> dsp_sources_registry;
@@ -16,8 +18,15 @@ namespace dsp
             if (remote)
                 if (source.first == "remote")
                     continue;
-            std::vector<SourceDescriptor> devices = source.second.getSources();
-            all_sources.insert(all_sources.end(), devices.begin(), devices.end());
+            try
+            {
+                std::vector<SourceDescriptor> devices = source.second.getSources();
+                all_sources.insert(all_sources.end(), devices.begin(), devices.end());
+            }
+            catch (std::exception &e)
+            {
+                logger->error("Error trying to list " + source.first + " sources! %s", e.what());
+            }
         }
 
         return all_sources;
@@ -28,7 +37,7 @@ namespace dsp
         for (std::pair<std::string, RegisteredSource> source : dsp_sources_registry)
             if (descriptor.source_type == source.first)
                 return source.second.getInstance(descriptor);
-        throw std::runtime_error("Could not find a handler for device " + descriptor.name);
+        throw satdump_exception("Could not find a handler for device " + descriptor.name);
     }
 
     void registerAllSources()
