@@ -8,8 +8,31 @@
 #include "resources.h"
 #include "common/widgets/stepped_slider.h"
 
+struct OverlayCache
+{
+    size_t width = 0;
+    size_t height = 0;
+    std::vector<size_t> map;
+};
+
+struct GrayscaleOverlayCache
+{
+    size_t width = 0;
+    size_t height = 0;
+    std::map<size_t, float> map;
+};
+
 class OverlayHandler
 {
+private:
+    OverlayCache map_cache;
+    OverlayCache shores_cache;
+    OverlayCache latlon_cache;
+    GrayscaleOverlayCache cities_cache;
+    GrayscaleOverlayCache qth_cache;
+
+    image::TextDrawer text_drawer;
+
 public:
     // Colors
     ImVec4 color_borders = {0, 1, 0, 1};
@@ -18,6 +41,10 @@ public:
     ImVec4 color_qth = {1, 0, 1, 1};
     ImVec4 color_latlon = {0, 0, 1, 1};
 
+    // QTH Label
+    std::string qth_label;
+    std::string last_qth_label = "";
+
     // Settings
     bool draw_map_overlay = false;
     bool draw_shores_overlay = false;
@@ -28,8 +55,12 @@ public:
     int cities_type = 0;
     int cities_size = 50;
     int cities_scale_rank = 3;
+    int last_cities_type = -1;
+    int last_cities_size = -1;
+    int last_cities_scale_rank = -1;
 
     void set_defaults();
+    void clear_cache();
 
 public:
     OverlayHandler()
@@ -39,7 +70,7 @@ public:
 
     int enabled();
     bool drawUI();
-    void apply(image::Image<uint16_t> &img, std::function<std::pair<int, int>(float, float, int, int)> &proj_func, float *step_cnt = nullptr);
+    void apply(image::Image &img, std::function<std::pair<int, int>(double, double, int, int)> proj_func, float *step_cnt = nullptr);
     nlohmann::json get_config();
     void set_config(nlohmann::json in, bool status = true);
 };
\ No newline at end of file
