@@ -3,6 +3,7 @@
 #include "common/image/image.h"
 #include "common/projection/thinplatespline.h"
 #include <memory>
+#include <functional>
 
 namespace satdump
 {
@@ -16,7 +17,7 @@ namespace satdump
         */
         struct WarpOperation
         {
-            image::Image<uint16_t> input_image;
+            image::Image input_image;
             std::vector<projection::GCP> ground_control_points;
             int output_width;
             int output_height;
@@ -34,7 +35,7 @@ namespace satdump
         */
         struct WarpResult
         {
-            image::Image<uint16_t> output_image;
+            image::Image output_image;
             projection::GCP top_left;
             projection::GCP top_right;
             projection::GCP bottom_right;
