@@ -4,6 +4,8 @@
 #include "common/image/image.h"
 #include <mutex>
 #include <vector>
+#include <string>
+#include <functional>
 
 class ImageViewWidget
 {
@@ -37,8 +39,7 @@ public:
 
     std::function<void(int x, int y)> mouseCallback = [](int, int) {}; // Function that can be used to handle mouse events
 
-    void update(image::Image<uint16_t> image);
-    void update(image::Image<uint8_t> image);
+    void update(image::Image &image);
     void draw(ImVec2 win_size);
 
     unsigned int getTextID() { return img_chunks.size() > 0 ? img_chunks[0].texture_id : 0; }
