@@ -2,6 +2,7 @@
 #include <array>
 #include <cstring>
 #include "snr_plot.h"
+#include "themed_widgets.h"
 
 namespace widgets
 {
@@ -9,11 +10,11 @@ namespace widgets
     {
         ImGui::Text("SNR (dB) : ");
         ImGui::SameLine();
-        ImGui::TextColored(snr > 2 ? snr > 10 ? IMCOLOR_SYNCED : IMCOLOR_SYNCING : IMCOLOR_NOSYNC, "%0.3f", snr);
+        ImGui::TextColored(snr > 2 ? snr > 10 ? style::theme.green : style::theme.orange : style::theme.red, "%0.3f", snr);
 
         ImGui::Text("Peak SNR (dB) : ");
         ImGui::SameLine();
-        ImGui::TextColored(peak_snr > 2 ? peak_snr > 10 ? IMCOLOR_SYNCED : IMCOLOR_SYNCING : IMCOLOR_NOSYNC, "%0.3f", peak_snr);
+        ImGui::TextColored(peak_snr > 2 ? peak_snr > 10 ? style::theme.green : style::theme.orange : style::theme.red, "%0.3f", peak_snr);
 
         std::memmove(&snr_history[0], &snr_history[1], (200 - 1) * sizeof(float));
         snr_history[200 - 1] = snr;
@@ -30,8 +31,9 @@ namespace widgets
 
         ImGui::Text("Avg SNR (dB) : ");
         ImGui::SameLine();
-        ImGui::TextColored(avg_snr > 2 ? avg_snr > 10 ? IMCOLOR_SYNCED : IMCOLOR_SYNCING : IMCOLOR_NOSYNC, "%0.3f", avg_snr);
+        ImGui::TextColored(avg_snr > 2 ? avg_snr > 10 ? style::theme.green : style::theme.orange : style::theme.red, "%0.3f", avg_snr);
 
-        ImGui::PlotLines("", snr_history, IM_ARRAYSIZE(snr_history), 0, "", 0.0f, 25.0f, ImVec2(200 * ui_scale, 50 * ui_scale));
+        widgets::ThemedPlotLines(style::theme.plot_bg.Value, "", snr_history, IM_ARRAYSIZE(snr_history), 0, "", 0.0f, 25.0f,
+            ImVec2(200 * ui_scale, 50 * ui_scale));
     }
 }
\ No newline at end of file
