@@ -6,6 +6,7 @@
 #include <filesystem>
 #include <thread>
 #include "core/config.h"
+#include "core/exception.h"
 
 namespace satdump
 {
@@ -58,7 +59,7 @@ namespace satdump
             PipelineModule module2 = steps[2].modules[0];
 
             if (modules_registry.count(module1.module_name) <= 0 || modules_registry.count(module2.module_name) <= 0)
-                throw std::runtime_error("Module " + module1.module_name + " or " + module2.module_name + " is not registered. Cancelling pipeline.");
+                throw satdump_exception("Module " + module1.module_name + " or " + module2.module_name + " is not registered. Cancelling pipeline.");
 
             nlohmann::json params1 = prepareParameters(module1.parameters, parameters);
             nlohmann::json params2 = prepareParameters(module2.parameters, parameters);
@@ -100,9 +101,9 @@ namespace satdump
                     uiCallListMutex->unlock();
                 }
 
-                std::thread module1_thread([m1]()
+                std::thread module1_thread([&m1]()
                                            { m1->process(); });
-                std::thread module2_thread([m2]()
+                std::thread module2_thread([&m2]()
                                            { m2->process(); });
 
                 if (module1_thread.joinable())
@@ -149,7 +150,7 @@ namespace satdump
             {
                 // Check module exists!
                 if (modules_registry.count(modStep.module_name) <= 0)
-                    throw std::runtime_error("Module " + modStep.module_name + " is not registered. Cancelling pipeline.");
+                    throw satdump_exception("Module " + modStep.module_name + " is not registered. Cancelling pipeline.");
 
                 nlohmann::json final_parameters = prepareParameters(modStep.parameters, parameters);
 
@@ -384,6 +385,12 @@ namespace satdump
 
     void loadPipelines(std::string filepath)
     {
+        if (!std::filesystem::exists(filepath))
+        {
+            logger->error("Couldn't load pipelines! Was trying : " + filepath);
+            exit(1);
+        }
+
         logger->info("Loading pipelines from " + filepath);
 
         std::vector<std::pair<std::string, std::string>> pipelinesToLoad;
@@ -411,6 +418,14 @@ namespace satdump
 
         for (std::pair<std::string, std::string> pipeline : pipelinesToLoad)
             loadPipeline(pipeline.first);
+
+        std::sort(pipelines.begin(), pipelines.end(), [](const Pipeline &l, const Pipeline &r)
+                  {
+                                                          std::string lname = l.readable_name;
+                                                          std::string rname = r.readable_name;
+                                                          std::transform(lname.begin(), lname.end(), lname.begin(), ::tolower);
+                                                          std::transform(rname.begin(), rname.end(), rname.begin(), ::tolower);
+                                                          return lname < rname; });
     }
 
     std::optional<Pipeline> getPipelineFromName(std::string downlink_pipeline)
