@@ -1,11 +1,12 @@
 #include "module_ccsds_conv_concat_decoder.h"
 #include "logger.h"
 #include "common/codings/differential/nrzm.h"
-#include "imgui/imgui.h"
+#include "common/widgets/themed_widgets.h"
 #include "common/codings/randomization.h"
+#include "core/exception.h"
 
 // Return filesize
-size_t getFilesize(std::string filepath);
+uint64_t getFilesize(std::string filepath);
 
 namespace ccsds
 {
@@ -15,12 +16,10 @@ namespace ccsds
 
           d_constellation_str(parameters["constellation"].get<std::string>()),
 
-          d_oqpsk_delay(parameters.count("oqpsk_delay") > 0 ? parameters["oqpsk_delay"].get<bool>() : false),
-          d_oqpsk_mode(parameters.count("oqpsk_mode") > 0 ? parameters["oqpsk_mode"].get<bool>() : false),
           d_iq_invert(parameters.count("iq_invert") > 0 ? parameters["iq_invert"].get<bool>() : false),
           d_cadu_size(parameters["cadu_size"].get<int>()),
           d_cadu_bytes(ceil(d_cadu_size / 8.0)), // If we can't use complete bytes, add one and padding
-          d_buffer_size(d_cadu_size),
+          d_buffer_size(std::max<int>(d_cadu_size, 8192)),
 
           d_viterbi_outsync_after(parameters["viterbi_outsync_after"].get<int>()),
           d_viterbi_ber_threasold(parameters["viterbi_ber_thresold"].get<float>()),
@@ -41,8 +40,9 @@ namespace ccsds
     {
         viterbi_out = new uint8_t[d_buffer_size * 8];
         soft_buffer = new int8_t[d_buffer_size];
-        frame_buffer = new uint8_t[d_cadu_size * 8]; // Larger by safety
+        frame_buffer = new uint8_t[d_buffer_size * 8]; // Larger by safety
         d_bpsk_90 = false;
+        d_oqpsk_mode = false;
 
         // Get constellation
         if (d_constellation_str == "bpsk")
@@ -57,8 +57,13 @@ namespace ccsds
         }
         else if (d_constellation_str == "qpsk")
             d_constellation = dsp::QPSK;
+        else if (d_constellation_str == "oqpsk")
+        {
+            d_constellation = dsp::QPSK;
+            d_oqpsk_mode = true;
+        }
         else
-            throw std::runtime_error("CCSDS Concatenated 1/2 Decoder : invalid constellation type!");
+            throw satdump_exception("CCSDS Concatenated 1/2 Decoder : invalid constellation type!");
 
         std::vector<phase_t> d_phases;
 
@@ -79,7 +84,7 @@ namespace ccsds
             else if (d_rs_type == "rs239")
                 rstype = reedsolomon::RS239;
             else
-                throw std::runtime_error("CCSDS Concatenated 1/2 Decoder : invalid Reed-Solomon type!");
+                throw satdump_exception("CCSDS Concatenated 1/2 Decoder : invalid Reed-Solomon type!");
         }
 
         // Parse sync marker if set
@@ -163,8 +168,6 @@ namespace ccsds
 
         diff::NRZMDiff diff;
 
-        int8_t last_q_oqpsk = 0; // For delaying OQPSK
-
         while (input_data_type == DATA_FILE ? !data_in.eof() : input_active.load())
         {
             // Read a buffer
@@ -173,17 +176,6 @@ namespace ccsds
             else
                 input_fifo->read((uint8_t *)soft_buffer, d_buffer_size);
 
-            // OQPSK Delay
-            if (d_oqpsk_delay)
-            {
-                for (int i = 0; i < d_buffer_size / 2; i++)
-                {
-                    int8_t back = soft_buffer[i * 2 + 0];
-                    soft_buffer[i * 2 + 0] = last_q_oqpsk;
-                    last_q_oqpsk = back;
-                }
-            }
-
             if (d_bpsk_90 || d_iq_invert) // Symbols are swapped for some Q/BPSK sats
                 rotate_soft((int8_t *)soft_buffer, d_buffer_size, PHASE_0, true);
 
@@ -251,7 +243,7 @@ namespace ccsds
                 lastTime = time(NULL);
                 std::string viterbi_state = viterbi_lock == 0 ? "NOSYNC" : "SYNCED";
                 std::string deframer_state = deframer->getState() == deframer->STATE_NOSYNC ? "NOSYNC" : (deframer->getState() == deframer->STATE_SYNCING ? "SYNCING" : "SYNCED");
-                logger->info("Progress " + std::to_string(round(((float)progress / (float)filesize) * 1000.0f) / 10.0f) + "%%, Viterbi : " + viterbi_state + " BER : " + std::to_string(viterbi_ber) + ", Deframer : " + deframer_state);
+                logger->info("Progress " + std::to_string(round(((double)progress / (double)filesize) * 1000.0f) / 10.0f) + "%%, Viterbi : " + viterbi_state + " BER : " + std::to_string(viterbi_ber) + ", Deframer : " + deframer_state);
             }
         }
 
@@ -284,14 +276,14 @@ namespace ccsds
                 ImDrawList *draw_list = ImGui::GetWindowDrawList();
                 draw_list->AddRectFilled(ImGui::GetCursorScreenPos(),
                                          ImVec2(ImGui::GetCursorScreenPos().x + 200 * ui_scale, ImGui::GetCursorScreenPos().y + 200 * ui_scale),
-                                         ImColor::HSV(0, 0, 0));
+                                         style::theme.widget_bg);
 
                 for (int i = 0; i < 2048; i++)
                 {
                     draw_list->AddCircleFilled(ImVec2(ImGui::GetCursorScreenPos().x + (int)(100 * ui_scale + (((int8_t *)soft_buffer)[i] / 127.0) * 130 * ui_scale) % int(200 * ui_scale),
                                                       ImGui::GetCursorScreenPos().y + (int)(100 * ui_scale + rng.gasdev() * 14 * ui_scale) % int(200 * ui_scale)),
                                                2 * ui_scale,
-                                               ImColor::HSV(113.0 / 360.0, 1, 1, 1.0));
+                                               style::theme.constellation);
                 }
 
                 ImGui::Dummy(ImVec2(200 * ui_scale + 3, 200 * ui_scale + 3));
@@ -301,14 +293,14 @@ namespace ccsds
                 ImDrawList *draw_list = ImGui::GetWindowDrawList();
                 draw_list->AddRectFilled(ImGui::GetCursorScreenPos(),
                                          ImVec2(ImGui::GetCursorScreenPos().x + 200 * ui_scale, ImGui::GetCursorScreenPos().y + 200 * ui_scale),
-                                         ImColor::HSV(0, 0, 0));
+                                         style::theme.widget_bg);
 
                 for (int i = 0; i < 2048; i++)
                 {
                     draw_list->AddCircleFilled(ImVec2(ImGui::GetCursorScreenPos().x + (int)(100 * ui_scale + (((int8_t *)soft_buffer)[i * 2 + 0] / 127.0) * 100 * ui_scale) % int(200 * ui_scale),
                                                       ImGui::GetCursorScreenPos().y + (int)(100 * ui_scale + (((int8_t *)soft_buffer)[i * 2 + 1] / 127.0) * 100 * ui_scale) % int(200 * ui_scale)),
                                                2 * ui_scale,
-                                               ImColor::HSV(113.0 / 360.0, 1, 1, 1.0));
+                                               style::theme.constellation);
                 }
 
                 ImGui::Dummy(ImVec2(200 * ui_scale + 3, 200 * ui_scale + 3));
@@ -327,18 +319,19 @@ namespace ccsds
                 ImGui::SameLine();
 
                 if (viterbi_lock == 0)
-                    ImGui::TextColored(IMCOLOR_NOSYNC, "NOSYNC");
+                    ImGui::TextColored(style::theme.red, "NOSYNC");
                 else
-                    ImGui::TextColored(IMCOLOR_SYNCED, "SYNCED");
+                    ImGui::TextColored(style::theme.green, "SYNCED");
 
                 ImGui::Text("BER   : ");
                 ImGui::SameLine();
-                ImGui::TextColored(viterbi_lock == 0 ? IMCOLOR_NOSYNC : IMCOLOR_SYNCED, UITO_C_STR(ber));
+                ImGui::TextColored(viterbi_lock == 0 ? style::theme.red : style::theme.green, UITO_C_STR(ber));
 
                 std::memmove(&ber_history[0], &ber_history[1], (200 - 1) * sizeof(float));
                 ber_history[200 - 1] = ber;
 
-                ImGui::PlotLines("", ber_history, IM_ARRAYSIZE(ber_history), 0, "", 0.0f, 1.0f, ImVec2(200 * ui_scale, 50 * ui_scale));
+                widgets::ThemedPlotLines(style::theme.plot_bg.Value, "", ber_history, IM_ARRAYSIZE(ber_history), 0, "", 0.0f, 1.0f,
+                                         ImVec2(200 * ui_scale, 50 * ui_scale));
             }
 
             ImGui::Spacing();
@@ -349,12 +342,19 @@ namespace ccsds
 
                 ImGui::SameLine();
 
-                if (deframer->getState() == deframer->STATE_NOSYNC)
-                    ImGui::TextColored(IMCOLOR_NOSYNC, "NOSYNC");
-                else if (deframer->getState() == deframer->STATE_SYNCING)
-                    ImGui::TextColored(IMCOLOR_SYNCING, "SYNCING");
+                if (viterbi_lock == 0)
+                {
+                    ImGui::TextColored(ImGui::GetStyleColorVec4(ImGuiCol_TextDisabled), "NOSYNC");
+                }
                 else
-                    ImGui::TextColored(IMCOLOR_SYNCED, "SYNCED");
+                {
+                    if (deframer->getState() == deframer->STATE_NOSYNC)
+                        ImGui::TextColored(style::theme.red, "NOSYNC");
+                    else if (deframer->getState() == deframer->STATE_SYNCING)
+                        ImGui::TextColored(style::theme.orange, "SYNCING");
+                    else
+                        ImGui::TextColored(style::theme.green, "SYNCED");
+                }
             }
 
             ImGui::Spacing();
@@ -368,12 +368,19 @@ namespace ccsds
                     {
                         ImGui::SameLine();
 
-                        if (errors[i] == -1)
-                            ImGui::TextColored(IMCOLOR_NOSYNC, "%i ", i);
-                        else if (errors[i] > 0)
-                            ImGui::TextColored(IMCOLOR_SYNCING, "%i ", i);
+                        if (viterbi_lock == 0 || deframer->getState() == deframer->STATE_NOSYNC)
+                        {
+                            ImGui::TextColored(ImGui::GetStyleColorVec4(ImGuiCol_TextDisabled), "%i ", i);
+                        }
                         else
-                            ImGui::TextColored(IMCOLOR_SYNCED, "%i ", i);
+                        {
+                            if (errors[i] == -1)
+                                ImGui::TextColored(style::theme.red, "%i ", i);
+                            else if (errors[i] > 0)
+                                ImGui::TextColored(style::theme.orange, "%i ", i);
+                            else
+                                ImGui::TextColored(style::theme.green, "%i ", i);
+                        }
                     }
                 }
             }
@@ -381,7 +388,7 @@ namespace ccsds
         ImGui::EndGroup();
 
         if (!streamingInput)
-            ImGui::ProgressBar((float)progress / (float)filesize, ImVec2(ImGui::GetWindowWidth() - 10, 20 * ui_scale));
+            ImGui::ProgressBar((double)progress / (double)filesize, ImVec2(ImGui::GetContentRegionAvail().x, 20 * ui_scale));
 
         ImGui::End();
     }
