@@ -4,9 +4,10 @@
 #include "imgui/imgui.h"
 #include "common/codings/randomization.h"
 #include "common/codings/differential/qpsk_diff.h"
+#include "core/exception.h"
 
 // Return filesize
-size_t getFilesize(std::string filepath);
+uint64_t getFilesize(std::string filepath);
 
 namespace ccsds
 {
@@ -49,7 +50,7 @@ namespace ccsds
         else if (d_constellation_str == "qpsk")
             d_constellation = dsp::QPSK;
         else
-            throw std::runtime_error("CCSDS Simple PSK Decoder : invalid constellation type!");
+            throw satdump_exception("CCSDS Simple PSK Decoder : invalid constellation type!");
 
         // Parse RS
         reedsolomon::RS_TYPE rstype = reedsolomon::RS223;
@@ -60,7 +61,7 @@ namespace ccsds
             else if (d_rs_type == "rs239")
                 rstype = reedsolomon::RS239;
             else
-                throw std::runtime_error("CCSDS Simple PSK Decoder : invalid Reed-Solomon type!");
+                throw satdump_exception("CCSDS Simple PSK Decoder : invalid Reed-Solomon type!");
         }
 
         // Parse sync marker if set
@@ -313,7 +314,7 @@ namespace ccsds
                     if (deframer_qpsk->getState() > deframer->getState()) // ...and use the one that is locked
                         def = deframer_qpsk;
                 std::string deframer_state = def->getState() == def->STATE_NOSYNC ? "NOSYNC" : (def->getState() == def->STATE_SYNCING ? "SYNCING" : "SYNCED");
-                logger->info("Progress " + std::to_string(round(((float)progress / (float)filesize) * 1000.0f) / 10.0f) + "%%, Deframer : " + deframer_state);
+                logger->info("Progress " + std::to_string(round(((double)progress / (double)filesize) * 1000.0) / 10.0) + "%%, Deframer : " + deframer_state);
             }
         }
 
@@ -340,14 +341,14 @@ namespace ccsds
                 ImDrawList *draw_list = ImGui::GetWindowDrawList();
                 draw_list->AddRectFilled(ImGui::GetCursorScreenPos(),
                                          ImVec2(ImGui::GetCursorScreenPos().x + 200 * ui_scale, ImGui::GetCursorScreenPos().y + 200 * ui_scale),
-                                         ImColor::HSV(0, 0, 0));
+                                         style::theme.widget_bg);
 
                 for (int i = 0; i < 2048; i++)
                 {
                     draw_list->AddCircleFilled(ImVec2(ImGui::GetCursorScreenPos().x + (int)(100 * ui_scale + (((int8_t *)soft_buffer)[i] / 127.0) * 130 * ui_scale) % int(200 * ui_scale),
                                                       ImGui::GetCursorScreenPos().y + (int)(100 * ui_scale + rng.gasdev() * 14 * ui_scale) % int(200 * ui_scale)),
                                                2 * ui_scale,
-                                               ImColor::HSV(113.0 / 360.0, 1, 1, 1.0));
+                                               style::theme.constellation);
                 }
 
                 ImGui::Dummy(ImVec2(200 * ui_scale + 3, 200 * ui_scale + 3));
@@ -357,14 +358,14 @@ namespace ccsds
                 ImDrawList *draw_list = ImGui::GetWindowDrawList();
                 draw_list->AddRectFilled(ImGui::GetCursorScreenPos(),
                                          ImVec2(ImGui::GetCursorScreenPos().x + 200 * ui_scale, ImGui::GetCursorScreenPos().y + 200 * ui_scale),
-                                         ImColor::HSV(0, 0, 0));
+                                         style::theme.widget_bg);
 
                 for (int i = 0; i < 2048; i++)
                 {
                     draw_list->AddCircleFilled(ImVec2(ImGui::GetCursorScreenPos().x + (int)(100 * ui_scale + (((int8_t *)soft_buffer)[i * 2 + 0] / 127.0) * 100 * ui_scale) % int(200 * ui_scale),
                                                       ImGui::GetCursorScreenPos().y + (int)(100 * ui_scale + (((int8_t *)soft_buffer)[i * 2 + 1] / 127.0) * 100 * ui_scale) % int(200 * ui_scale)),
                                                2 * ui_scale,
-                                               ImColor::HSV(113.0 / 360.0, 1, 1, 1.0));
+                                               style::theme.constellation);
                 }
 
                 ImGui::Dummy(ImVec2(200 * ui_scale + 3, 200 * ui_scale + 3));
@@ -383,11 +384,11 @@ namespace ccsds
                 ImGui::SameLine();
 
                 if (def->getState() == def->STATE_NOSYNC)
-                    ImGui::TextColored(IMCOLOR_NOSYNC, "NOSYNC");
+                    ImGui::TextColored(style::theme.red, "NOSYNC");
                 else if (def->getState() == def->STATE_SYNCING)
-                    ImGui::TextColored(IMCOLOR_SYNCING, "SYNCING");
+                    ImGui::TextColored(style::theme.orange, "SYNCING");
                 else
-                    ImGui::TextColored(IMCOLOR_SYNCED, "SYNCED");
+                    ImGui::TextColored(style::theme.green, "SYNCED");
             }
 
             ImGui::Spacing();
@@ -401,12 +402,19 @@ namespace ccsds
                     {
                         ImGui::SameLine();
 
-                        if (errors[i] == -1)
-                            ImGui::TextColored(IMCOLOR_NOSYNC, "%i ", i);
-                        else if (errors[i] > 0)
-                            ImGui::TextColored(IMCOLOR_SYNCING, "%i ", i);
+                        if (def->getState() == def->STATE_NOSYNC)
+                        {
+                            ImGui::TextColored(ImGui::GetStyleColorVec4(ImGuiCol_TextDisabled), "%i ", i);
+                        }
                         else
-                            ImGui::TextColored(IMCOLOR_SYNCED, "%i ", i);
+                        {
+                            if (errors[i] == -1)
+                                ImGui::TextColored(style::theme.red, "%i ", i);
+                            else if (errors[i] > 0)
+                                ImGui::TextColored(style::theme.orange, "%i ", i);
+                            else
+                                ImGui::TextColored(style::theme.green, "%i ", i);
+                        }
                     }
                 }
             }
@@ -414,7 +422,7 @@ namespace ccsds
         ImGui::EndGroup();
 
         if (!streamingInput)
-            ImGui::ProgressBar((float)progress / (float)filesize, ImVec2(ImGui::GetWindowWidth() - 10, 20 * ui_scale));
+            ImGui::ProgressBar((double)progress / (double)filesize, ImVec2(ImGui::GetContentRegionAvail().x, 20 * ui_scale));
 
         ImGui::End();
     }
