@@ -9,18 +9,13 @@
 
 namespace satdump
 {
-    extern std::string error_message;
-    extern bool isError_UI;
-
+    extern bool update_ui;
     extern ctpl::thread_pool ui_thread_pool;
 
-    extern bool light_theme;
-
     extern std::shared_ptr<RecorderApplication> recorder_app;
     extern std::shared_ptr<ViewerApplication> viewer_app;
 
-    void initMainUI(float device_scale);
-    void updateUI(float device_scale);
+    void initMainUI();
     void exitMainUI();
-    void renderMainUI(int wwidth, int wheight);
-}
\ No newline at end of file
+    void renderMainUI();
+}
